from urllib.parse import urljoin
import logging

from geosyspy.utils.constants import *
from geosyspy.utils.helper import Helper
from geosyspy.utils.http_client import *
from datetime import datetime


class MasterDataManagementService:

    def __init__(self, base_url: str, http_client: HttpClient):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.logger = logging.getLogger(__name__)

    def create_season_field_id(self, polygon: str) -> object:
        """Posts the payload below to the master data management endpoint.

        This method returns a season field id. The season field id is required
        to request other APIs endpoints.

        Args:
            polygon : A string representing a polygon.

        Returns:
            A response object.

        """

        year = datetime.now().year
        payload = {
            "Geometry": polygon,
            "Crop": {"Id": "CORN"},
            "SowingDate": f"{year}-01-01",
        }
        mdm_url: str = urljoin(self.base_url, GeosysApiEndpoints.MASTER_DATA_MANAGEMENT_ENDPOINT.value+"/seasonfields")
        return self.http_client.post(mdm_url, payload)

    def extract_season_field_id(self, polygon: str) -> str:
        """Extracts the season field id from the response object.

        Args:
            polygon : A string representing a polygon.

        Returns:
            A string representing the season field id.

        Raises:
            ValueError: The response status code is not as expected.
        """

        response = self.create_season_field_id(polygon)
        dict_response = response.json()

        if response.status_code == 400 and "sowingDate" in dict_response["errors"]["body"]:

            text: str = dict_response["errors"]["body"]["sowingDate"][0]["message"]
            return Helper.get_matched_str_from_pattern(SEASON_FIELD_ID_REGEX, text)

        elif response.status_code == 201:
            return dict_response["id"]
        else:
            raise ValueError(
                f"Cannot handle HTTP response : {str(response.status_code)} : {str(response.json())}"
            )

    def get_season_field_unique_id(self, season_field_id: str) -> str:
        """Extracts the season field unique id from the response object.

        Args:
            season_field_id : A string representing the seasonfield legacy na id

        Returns:
            A string representing the season field unique id.

        Raises:
            ValueError: The response status code is not as expected.
        """

        mdm_url: str = urljoin(self.base_url, GeosysApiEndpoints.MASTER_DATA_MANAGEMENT_ENDPOINT.value + f"/seasonfields/{season_field_id}?$fields=externalids")

        response = self.http_client.get(mdm_url)

        dict_response = response.json()

        # extract unique id from response:
        if response.status_code == 200:
            return dict_response["externalIds"]["id"]
        else:
            raise ValueError(
                f"Cannot handle HTTP response : {str(response.status_code)} : {str(response.json())}"
            )


    def get_available_crops_code(self) -> [str]:
        """Extracts the list of available crops for the connected user

        Args:

        Returns:
            A list of string representing the available crop codes

        Raises:
            ValueError: The response status code is not as expected.
        """
        mdm_url: str = urljoin(self.base_url, GeosysApiEndpoints.MASTER_DATA_MANAGEMENT_ENDPOINT.value + f"/crops?$fields=code&$limit=none")

        response = self.http_client.get(mdm_url)

        dict_response = response.json()

        if response.status_code == 200:
            return dict_response
        else:
            raise ValueError(
                f"Cannot handle HTTP response : {str(response.status_code)} : {str(response.json())}"
            )


    def get_permission_codes(self) -> [str]:
        """Extracts the list of available permissions for the connected user

        Args:

        Returns:
            A list of string representing the available permissions codes

        Raises:
            ValueError: The response status code is not as expected.
        """
        mdm_url: str = urljoin(self.base_url, GeosysApiEndpoints.MASTER_DATA_MANAGEMENT_ENDPOINT.value + f"/profile?$fields=permissions&$limit=none")

        response = self.http_client.get(mdm_url)

        dict_response = response.json()

        if response.status_code == 200:
            return dict_response
        else:
            raise ValueError(
                f"Cannot handle HTTP response : {str(response.status_code)} : {str(response.json())}"
            )





from enum import Enum


class ProcessorConfiguration(Enum):
    """
    Available different Geosys Api Processor key path & profile values
    """
    PLANTED_AREA = {"profile":"planted_area_default", "api_processor_path":"planted_area"}
    POTENTIAL_SCORE = {"profile":"potential_score_default", "api_processor_path":"potential_score"}
    BRAZIL_IN_SEASON_CROP_ID = {"profile":"brazil_in_season_crop_id_default", "api_processor_path":"brazil_in_season_crop_id"}
    GREENNESS = {"profile":"greenness_default", "api_processor_path":"greenness"}
    HARVEST_READINESS = {"profile":"harvest_readiness_default", "api_processor_path":"harvest_readiness"}
    ZARC = {"profile":"zarc_default", "api_processor_path":"zarc"}
    EMERGENCE_IN_SEASON = {"profile":"inseason-emergence_default", "api_processor_path":"emergence-date"}
    EMERGENCE_HISTORICAL = {"profile":"historical-emergence_default", "api_processor_path":"historical-emergence"}
    EMERGENCE_DELAY = {"profile": "inseason-emergence_default", "api_processor_path": "emergence-delay"}
    HARVEST_IN_SEASON = {"profile":"inseason-harvest_default", "api_processor_path":"inseason-harvest"}
    HARVEST_HISTORICAL = {"profile":"historical-harvest_default", "api_processor_path":"historical-harvest"}
    MRTS = {"profile":"mrts_default", "api_processor_path":"mrts"}




"""
This file contains python dictionaries used to store the Geosys' APIs
URLs for the preprod/prod environments.

"""

IDENTITY_URLS = {
    'na': {
        'preprod': 'https://identity.preprod.geosys-na.com/v2.1/connect/token',
        'prod': 'https://identity.geosys-na.com/v2.1/connect/token'
    }
}
GEOSYS_API_URLS = {
    'na': {
        'preprod': 'https://api-pp.geosys-na.net',
        'prod': 'https://api.geosys-na.net'
    }
}
GIS_API_URLS = {
    'na': {
        'preprod': 'https://gis-services-pp.geosys.com',
        'prod': 'https://gis-services.geosys.com'
    }
}



import logging
import requests_mock
import requests
import os


def load_data_from_textfile(file_name):
    # valid path for all os system
    file_path = os.path.join(os.path.dirname(__file__), f"resources/{file_name}").replace("\\", "/")
    if os.path.isfile(file_path):
        with open(file_path, 'r') as data:
            return data.read()
    else:
        logging.error(f"Unable to read file: {file_name}")


def load_binary_data_from_zipfile(file_name):
    # valid path for all os system
    file_path = os.path.join(os.path.dirname(__file__), f"resources/{file_name}").replace("\\", "/")
    if os.path.isfile(file_path):
        with open(file_path, 'rb') as data:
            return data.read()
    else:
        logging.error(f"Unable to read zip file: {file_name}")


def mock_http_response_text_content(method, content=None, status_code=200):
    if method == "GET":
        with requests_mock.Mocker() as m:
            m.get('http://geosys.com', text=content)
            return requests.get('http://geosys.com')
    if method == "POST":
        with requests_mock.Mocker() as m:
            m.post('http://geosys.com', text=content, status_code= status_code)
            return requests.post('http://geosys.com' )
    if method == "PATCH":
        with requests_mock.Mocker() as m:
            m.patch('http://geosys.com', text=content)
            return requests.patch('http://geosys.com')


def mock_http_response_binary_content(method, binary_content=None):
    if method == "GET":
        with requests_mock.Mocker() as m:
            m.get(url='http://geosys.com', content=binary_content)
            return requests.get(url='http://geosys.com')



import json
import logging
import pandas as pd
from datetime import datetime, timedelta
from urllib.parse import urljoin
from geosyspy.utils.constants import *
from geosyspy.utils.http_client import *


class AgriquestService:

    def __init__(self, base_url: str, http_client: HttpClient):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.logger = logging.getLogger(__name__)

    def weather_indicators_builder(self, start_date, end_date, isFrance):
        """build weather indicators list from 2 dates

            Args:
                start_date (datetime) : the start date used for the request
                end_date (datetime) : the end date used for the request
                isFrance (boolean) : resuqest mad for France or not

            Returns:
                A list of int (weather indicators)
                - 2 : ENS observed data
                - 3 : Arome Weather Observed data (Meteo France, France only)
                - 4 : ECMWF Weather  Forecast Data
                - 5 : GFS Weather Forecast Data

            """

        today = datetime.now().date()
        result = []

        # Forecast indicators
        if end_date > today:
            result.extend([4, 5])

        if start_date < today:
            if not isFrance:
                result.append(2)
            else:
                result.append(3)

        return result

    def is_block_for_france(self, block_code: AgriquestBlocks):
        """
        method to check if the services block code is dedicated to France

            Args:
                 block_code (AgriquestBlocks): the block code to check
            Returns:
                boolean
        """
        if (block_code.value in [item.value for item in AgriquestFranceBlockCode]):
            return True
        return False

    def get_agriquest_block_weather_data(self,
                                         start_date: str,
                                         end_date: str,
                                         block_code: AgriquestBlocks,
                                         indicator_list: [int],
                                         weather_type: AgriquestWeatherType
                                         ):
        """
            method to call Weather AgriQuest Api and build a panda DataFrame

                Args:
                     start_date (str),
                     end_date (str),
                     block_code (AgriquestBlocks): the AQ block to check ,
                     indicator_list ([int]): list of weather indicator types,
                     weather_type(AgriquestWeatherType): type of weather data to retrieve
                Returns:
                    Panda DataFrame representing the value corresponding to the weather_type of each AMU of the block,
                    on the specified period provided in parameter.
        """

        payload = {
            "analyticName": weather_type.name,
            "commodityId": AgriquestCommodityCode.ALL_VEGETATION.value,
            "startDate": start_date,
            "endDate": end_date,
            "idPixelType": 1,
            "idBlock": block_code.value,
            "indicatorTypeIds": indicator_list
        }
        parameters: str = f"/{weather_type.value}/export-map/year-of-interest"
        aq_url: str = urljoin(self.base_url, GeosysApiEndpoints.AGRIQUEST_ENDPOINT.value + parameters)
        response = self.http_client.post(aq_url, payload)
        if response.status_code == 200:
            dict_response = response.json()
            df = pd.read_json(json.dumps(dict_response))

            # Ignore first line
            df = df.iloc[1:]

            # first line as column header
            df.columns = df.iloc[0]

            # Ignore first line after using columns header
            df = df.iloc[1:]

            df = df.rename(columns={"Name": "AMU"})
            return df
        else:
            self.logger.info(response.status_code)


    def get_agriquest_block_ndvi_data(self,
                                      date: str,
                                      block_code: AgriquestBlocks,
                                      commodity: AgriquestCommodityCode,
                                      indicator_list: [int]
                                      ):
        """
            method to call year-of-interest AgriQuest Api and build a panda DataFrame

                Args:
                     date (str),
                     block_code (AgriquestBlocks): the AQ block to check ,
                     indicator_list ([int]): list of indicator types
                Returns:
                    Panda DataFrame representing the value corresponding to NDVI index of each AMU of the block,
                    on the specified date (dayOfMeasure) provided in parameter.
        """

        payload = {
            "analyticName": "NDVI",
            "commodityId": commodity.value,
            "dayOfMeasure": date,
            "idPixelType": 1,
            "idBlock": block_code.value,
            "indicatorTypeIds": indicator_list
        }
        parameters: str = f"/vegetation-vigor-index/export-map/year-of-interest"
        aq_url: str = urljoin(self.base_url, GeosysApiEndpoints.AGRIQUEST_ENDPOINT.value + parameters)
        response = self.http_client.post(aq_url, payload)
        if response.status_code == 200:
            dict_response = response.json()
            df = pd.read_json(json.dumps(dict_response))

            # Ignore first line
            df = df.iloc[1:]

            # first line as column header
            df.columns = df.iloc[0]

            # Ignore first line after using columns header
            df = df.iloc[1:]

            df = df.rename(columns={"Name": "AMU", "Value": "NDVI"})
            return df
        else:
            self.logger.info(response.status_code)



import datetime as dt
import numpy as np
from unittest.mock import patch

from geosyspy.services.map_product_service import MapProductService
from geosyspy.utils.http_client import *
from tests.test_helper import *
from geosyspy.utils.constants import *


class TestMapProductService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                             "client_secret_123456",
                             "username_123",
                             "password_123",
                             "preprod",
                             "na")
    priority_queue = "realtime"

    service = MapProductService(base_url=url, http_client=http_client,priority_queue=priority_queue)

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_satellite_coverage(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "satellite_coverage_image_references_mock_http_response"))
        start_date = dt.datetime.strptime("2022-01-01", "%Y-%m-%d")
        end_date = dt.datetime.strptime("2023-01-01", "%Y-%m-%d")
        info = self.service.get_satellite_coverage(
            "fakeSeasonFieldId", start_date, end_date, "NDVI", [SatelliteImageryCollection.SENTINEL_2]
        )

        assert {"coverageType", "image.id", "image.availableBands", "image.sensor", "image.soilMaterial",
                "image.spatialResolution", "image.weather", "image.date", "seasonField.id"}.issubset(set(info.columns))






from unittest.mock import patch

from geosyspy.services.analytics_processor_service import AnalyticsProcessorService
from geosyspy.utils.constants import *
from geosyspy.utils.http_client import *
from tests.test_helper import *

geometry = "POLYGON((-91.17523978603823 40.29787117039518,-91.17577285022956 40.29199489606421,-91.167613719932 40.29199489606421,-91.1673028670095 40.29867040193312,-91.17523978603823 40.29787117039518))"


class TestAnalyticsProcessorService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")
    service = AnalyticsProcessorService(base_url=url, http_client=http_client)

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_task_status(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "processor_event_data_mock_http_response"))

        task_status = self.service.wait_and_check_task_status("task_id")
        assert task_status == "Ended"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_planted_area_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_planted_area_processor(start_date='2020-01-01', end_date='2021-01-01', seasonfield_id= 'seasonfieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_zarc_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_zarc_processor(start_date_emergence ='2020-01-01',
                                                     municipio=123,
                                                     soil_type=ZarcSoilType.SOIL_TYPE_1.value,
                                                     nb_days_sowing_emergence=50,
                                                     crop="Corn",
                                                     end_date_emergence='2021-01-01',
                                                     cycle= ZarcCycleType.CYCLE_TYPE_1.value,
                                                     seasonfield_id='seasonFieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_greenness_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_greenness_processor(start_date='2020-01-01',
                                                          crop="Corn",
                                                          end_date='2021-01-01',
                                                          sowing_date='2020-01-01',
                                                          geometry=geometry,
                                                          seasonfield_id='seasonFieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_emergence_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_emergence_processor(season_start_day='2020-01-01',
                                                          crop="Corn",
                                                          year=2021,
                                                          emergence_type=Emergence.EMERGENCE_IN_SEASON,
                                                          season_duration=110,
                                                          season_start_month=10,
                                                          geometry=geometry,
                                                          seasonfield_id='seasonFieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_harvest_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_harvest_processor(season_start_day='2020-01-01',
                                                        year=2021,
                                                        crop ='Corn',
                                                        season_duration=110,
                                                        season_start_month=10,
                                                        harvest_type=Harvest.HARVEST_HISTORICAL,
                                                        geometry=geometry,
                                                        seasonfield_id='seasonFieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_harvest_readiness_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_harvest_readiness_processor(start_date='2020-01-01',
                                                                  end_date='2020-01-01',
                                                                  sowing_date='2020-01-01',
                                                                  crop ='Corn',
                                                                  geometry=geometry,
                                                                  seasonfield_id='seasonFieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_brazil_in_season_crop_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_brazil_in_season_crop_id_processor(start_date='2020-01-01',
                                                                         end_date='2020-01-01',
                                                                         geometry=geometry,
                                                                         season=2021,
                                                                         seasonfield_id='seasonFieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_launch_potential_score_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_potential_score_processor(end_date='2021-01-01',
                                                                season_start_month=10,
                                                                season_duration=120,
                                                                season_start_day=30,
                                                                sowing_date='2020-01-01',
                                                                crop='Corn',
                                                                nb_historical_years=5,
                                                                geometry=geometry,
                                                                seasonfield_id='seasonFieldFakeId')
        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_mr_time_series_processor(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "launch_processor_data_mock_http_response"))

        task_id = self.service.launch_mr_time_series_processor(start_date="2020-10-09",
                                                               end_date="2022-10-09",
                                                               list_sensors=["Sentinel_2", "Landsat_8"],
                                                               denoiser=True,
                                                               smoother="ww",
                                                               eoc=True,
                                                               aggregation="mean",
                                                               index="ndvi",
                                                               raw_data=True,
                                                               polygon="POLYGON ((-0.49881816 46.27330504, -0.49231649 46.27320122, -0.49611449 46.26983426, -0.49821735 46.27094671, -0.49881816 46.27330504))")

        assert task_id == "cb58faaf8a5640e4913d16bfde3f5bbf"

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_s3_path_from_task_and_processor(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "processor_event_data_mock_http_response"))

        s3_path = self.service.get_s3_path_from_task_and_processor(task_id="4d0980e07b7245d49419ff5ec87fff09",
                                                                   processor_name="mrts")

        assert s3_path == "s3://geosys-geosys-us/2tKecZgMyEP6EkddLxa1gV/mrts/4d0980e07b7245d49419ff5ec87fff09"



class ImageReference:

    def __init__(self, image_id, image_date, image_sensor, season_field_id):
        self.image_id = image_id
        self.image_date = image_date
        self.image_sensor = image_sensor
        self.season_field_id = season_field_id

    def __str__(self):
        return f"{self.image_date} - {self.image_id} - {self.image_sensor}"



from . import oauth2_client
from oauthlib.oauth2 import TokenExpiredError
from requests_oauthlib import OAuth2Session


def renew_access_token(func):
    """Decorator used to wrap the Geosys class's http methods.

    This decorator wraps the geosys http methods (get,post...) and checks
    whether the used token is still valid or not. If not, it fetches a new token and
    uses it to make another request.

    """

    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except TokenExpiredError:
            self.access_token = self.__client_oauth.get_refresh_token()
            return func(self, *args, **kwargs)

    return wrapper


class HttpClient:

    def __init__(
            self,
            client_id: str,
            client_secret: str,
            username: str,
            password: str,
            enum_env: str,
            enum_region: str,
            bearer_token: str = None
    ):
        self.__client_oauth = oauth2_client.Oauth2Api(
            client_id=client_id,
            client_secret=client_secret,
            password=password,
            username=username,
            enum_env=enum_env,
            enum_region=enum_region,
            bearer_token=bearer_token
        )
        self.access_token = self.__client_oauth.token

        self.__client = OAuth2Session(self.__client_oauth.client_id, token=self.__client_oauth.token)

    @renew_access_token
    def get(self, url_endpoint: str, headers={}):
        """Gets the url_endpopint.

        Args:
            url_endpoint : A string representing the url to get.

        Returns:
            A response object.
        """
        return self.__client.get(url_endpoint, headers=headers)

    @renew_access_token
    def post(self, url_endpoint: str, payload: dict, headers={}):
        """Posts payload to the url_endpoint.

        Args:
            url_endpoint : A string representing the url to post paylaod to.
            payload : A python dict representing the payload.

        Returns:
            A response object.
        """
        return self.__client.post(url_endpoint, json=payload, headers=headers)

    @renew_access_token
    def patch(self, url_endpoint: str, payload: dict):
        """Patchs payload to the url_endpoint.

        Args:
            url_endpoint : A string representing the url to patch paylaod to.
            payload : A python dict representing the payload.

        Returns:
            A response object.
        """
        return self.__client.patch(url_endpoint, json=payload)

    def get_access_token(self):
        return self.access_token



from unittest.mock import patch
from geosyspy.utils.http_client import *



@patch('geosyspy.utils.http_client.HttpClient.get')
def test_get_request_from_api_should_success(get_response):
    get_response.return_value = "HTTP 200 OK"
    client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")

    response = client.get(url_endpoint="http://geosys.com", headers={})
    assert client.get.call_count == 1
    assert response == "HTTP 200 OK"


@patch('geosyspy.utils.http_client.HttpClient.post')
def test_post_request_should_success(post_response):
    post_response.return_value = "HTTP 201 OK"
    client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")
    payload = {
        "Geometry": "polygon",
        "Crop": {"Id": "CORN"},
        "SowingDate": "2022-01-01",
    }
    response = client.post(url_endpoint="http://geosys.com", payload=payload, headers={})
    assert client.post.call_count == 1
    assert response == "HTTP 201 OK"


@patch('geosyspy.utils.http_client.HttpClient.patch')
def test_patch_request_should_success(patch_response):
    patch_response.return_value = "HTTP 200 OK"
    client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")
    payload = {
        "Geometry": "new polygon",
        "Crop": {"Id": "CORN"},
        "SowingDate": "2022-01-01",
    }
    response = client.patch(url_endpoint="http://geosys.com", payload=payload)
    assert client.patch.call_count == 1
    assert response == "HTTP 200 OK"



import json
import logging
import pandas as pd
from datetime import datetime
from urllib.parse import urljoin
from geosyspy.utils.constants import *
from geosyspy.utils.http_client import *


class VegetationTimeSeriesService:

    def __init__(self, base_url: str, http_client: HttpClient):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.logger = logging.getLogger(__name__)

    def get_modis_time_series(self, season_field_id:str,
                              start_date:datetime,
                              end_date:datetime,
                              indicator:str):
        """Returns a pandas DataFrame.

        This method returns a time series of 'indicator' within the range
        'start_date' -> 'end_date' as a pandas DataFrame :

                     | index     | value |

            date     | indicator |   1   |
        __________________________________
         start-date  | indicator |   2   |

            ...      |    ...    |  ...  |

         end-date    | indicator |   8   |

        Args:
            season_field_id : A string representing the season_field_id.
            start_date : A datetime object representing the start date of the date interval the user wants to filter on.
            end_date : A datetime object representing the final date of the date interval the user wants to filter on.
            indicator : A string representing the indicator whose time series the user wants.

        Returns:
            df : A Pandas DataFrame containing two columns : index and value, and an index called 'date'.

        """

        self.logger.info("Calling APIs for aggregated time series")
        start_date: str = start_date.strftime("%Y-%m-%d")
        end_date: str = end_date.strftime("%Y-%m-%d")
        parameters: str = f"/values?$offset=0&$limit=None&$count=false&SeasonField.Id={season_field_id}&index={indicator}&$filter=Date >= '{start_date}' and Date <= '{end_date}'"
        vts_url: str = urljoin(self.base_url, GeosysApiEndpoints.VTS_ENDPOINT.value + parameters)
        response = self.http_client.get(vts_url)

        if response.status_code == 200:
            dict_response = response.json()
            df = pd.read_json(json.dumps(dict_response))
            df.set_index("date", inplace=True)
            return df
        else:
            self.logger.info(response.status_code)


    def get_time_series_by_pixel(self, season_field_id: str,
                                   start_date: datetime,
                                   end_date: datetime,
                                   indicator: str) -> pd.DataFrame:
            """Returns a pandas DataFrame.

            This method returns a time series of 'indicator' by pixel within the range 'start_date' -> 'end_date'
            as well as the pixel's coordinates X,Y in the MODIS's sinusoidal projection as a pandas DataFrame :



                            | index     | value | pixel.id | X | Y |
                            _______________________________________|
                date        | indicator |       |          |   |   |
                ___________________________________________________|

                start-date  | indicator |   2   |    1     |   |   |

                ...         | ...       |  ...  |   ...    |   |   |

                end-date    | indicator |   8   |   1000   |   |   |

            Args:
                polygon : A string representing a polygon.
                start_date : A datetime object representing the start date of the date interval the user wants to filter on.
                end_date : A datetime object representing the final date of the date interval the user wants to filter on.
                indicator : A string representing the indicator whose time series the user wants.

            Returns:
                df : A Pandas DataFrame containing five columns : index, value, pixel.id, X, Y and an index called 'date'.

            """

            self.logger.info("Calling APIs for time series by the pixel")
            start_date: str = start_date.strftime("%Y-%m-%d")
            end_date: str = end_date.strftime("%Y-%m-%d")
            parameters: str = f"/values?$offset=0&$limit=None&$count=false&SeasonField.Id={season_field_id}&index={indicator}&$filter=Date >= '{start_date}' and Date <= '{end_date}'"
            vts_url: str = urljoin(self.base_url, GeosysApiEndpoints.VTS_BY_PIXEL_ENDPOINT.value + parameters)
            # PSX/PSY : size in meters of one pixel
            # MODIS_GRID_LENGTH : theoretical length of the modis grid in meters
            # MODIS_GRID_HEIGHT : theoretical height of the modis grid in meters
            PSX = 231.65635826
            PSY = -231.65635826
            MODIS_GRID_LENGTH = 4800 * PSX * 36
            MODIS_GRID_HEIGHT = 4800 * PSY * 18

            response = self.http_client.get(vts_url)

            if response.status_code == 200:
                df = pd.json_normalize(response.json())
                df.set_index("date", inplace=True)

                # Extracts h, v, i and j from the pixel dataframe
                self.logger.info("Computing X and Y coordinates per pixel... ")
                df["h"] = df["pixel.id"].str.extract(r"h(.*)v").astype(int)
                df["v"] = df["pixel.id"].str.extract(r"v(.*)i").astype(int)
                df["i"] = df["pixel.id"].str.extract(r"i(.*)j").astype(int)
                df["j"] = df["pixel.id"].str.extract(r"j(.*)$").astype(int)

                # XUL/YUL : The coordinates of the top left corner of the tile h,v's top left pixel
                #  X/Y : the coordinates of the top left corner of the i,j pixel
                df["XUL"] = (df["h"] + 1) * 4800 * PSX - MODIS_GRID_LENGTH / 2
                df["YUL"] = (df["v"] + 1) * 4800 * PSY + MODIS_GRID_HEIGHT / 2
                df["X"] = df["i"] * PSX + df["XUL"]
                df["Y"] = df["j"] * PSY + df["YUL"]
                self.logger.info("Done ! ")
                return df[["index", "value", "pixel.id", "X", "Y"]]
            else:
                self.logger.info(response.status_code)


from datetime import datetime
from unittest.mock import patch
from geosyspy import Geosys
from dotenv import load_dotenv
import datetime as dt
import numpy as np
from geosyspy.utils.constants import *
from tests.test_helper import *

# read .env file
load_dotenv()

API_CLIENT_ID = os.getenv("API_CLIENT_ID")
API_CLIENT_SECRET = os.getenv("API_CLIENT_SECRET")
API_USERNAME = os.getenv("API_USERNAME")
API_PASSWORD = os.getenv("API_PASSWORD")

# polygon with two pixels : mh11v4i225j4612, mh11v4i226j4612
POLYGON = "POLYGON((-91.29152885756007 40.39177489815265,-91.28403789132507 40.391776131485386,-91.28386736508233 " \
          "40.389390758655935,-91.29143832829979 40.38874592864832,-91.29152885756007 40.39177489815265))"

class TestGeosys:
    client = Geosys(API_CLIENT_ID,
                    API_CLIENT_SECRET,
                    API_USERNAME,
                    API_PASSWORD,
                    Env.PREPROD,
                    Region.NA
                    )

    def test_authenticate(self):
        credentials = self.client.http_client.get_access_token();
        assert {"access_token", "expires_in", "token_type", "scope", "expires_at",
                "refresh_token"}.issubset(set(credentials.keys()))
        assert credentials['access_token'] is not None
        assert credentials['refresh_token'] is not None
        assert credentials['expires_at'] > datetime.today().timestamp()


    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_time_series_weather_historical_daily(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "time_series_weather_historical_daily_mock_http_response"))
        start_date = dt.datetime.strptime("2021-01-01", "%Y-%m-%d")
        end_date = dt.datetime.strptime("2022-01-01", "%Y-%m-%d")
        indicators = [

            "Precipitation",
            "Temperature",
            "Temperature.Ground",
            "Temperature.Standard",
            "Temperature.StandardMax",
            "Location",
            "Date",
        ]

        df = self.client.get_time_series(
            POLYGON,
            start_date,
            end_date,
            WeatherTypeCollection.WEATHER_HISTORICAL_DAILY,
            indicators,
        )

        assert {"precipitation.cumulative", "temperature.standard", "temperature.standardMax",
                "Location"}.issubset(set(df.columns))
        assert df.index.name == "date"
        assert df["weatherType"].iloc[1] == "HISTORICAL_DAILY"

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_metrics(self, get_response):

        fake_master_data_management_response = mock_http_response_text_content("GET", load_data_from_textfile(
            "master_data_management_get_unique_id_mock_http_response"))
        fake_analytics_fabric_response = mock_http_response_text_content("GET", load_data_from_textfile(
            "metrics_lai_radar_mock_http_response"))
        get_response.side_effect= [fake_master_data_management_response, fake_analytics_fabric_response]


        lai_radar_polygon = "POLYGON((-52.72591542 -18.7395779,-52.72604885 -18.73951122,-52.72603114 -18.73908689,-52.71556835 -18.72490316,-52.71391916 -18.72612966,-52.71362802 -18.72623726,-52.71086473 -18.72804231,-52.72083542 -18.74173696,-52.72118937 -18.74159174,-52.72139229 -18.7418552,-52.72600257 -18.73969719,-52.72591542 -18.7395779))"
        schema_id = "LAI_RADAR"
        start_date = dt.datetime.strptime("2023-01-02", "%Y-%m-%d")
        end_date = dt.datetime.strptime("2023-05-02", "%Y-%m-%d")
        df = self.client.get_metrics(lai_radar_polygon, schema_id, start_date, end_date)

        assert {"Values.RVI", "Values.LAI", "Schema.Id"}.issubset(set(df.columns))
        assert {"2023-01-02T00:00:00Z", "2023-01-03T00:00:00Z", "2023-01-14T00:00:00Z", "2023-02-25T00:00:00Z",
                "2023-03-26T00:00:00Z", "2023-04-27T00:00:00Z", "2023-05-02T00:00:00Z"}.issubset(set(df.index))
        assert df.index.name == "date"

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_time_series_weather_forecast_daily(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "time_series_weather_forecast_daily_mock_http_response"))
        start_date = dt.datetime.strptime("2021-01-01", "%Y-%m-%d")
        end_date = dt.datetime.strptime("2022-01-01", "%Y-%m-%d")
        indicators = [
            "Precipitation",
            "Temperature.Standard",
            "Temperature.StandardMax",
            "Date",
            "Location",
            "WeatherType"
        ]

        df = self.client.get_time_series(
            POLYGON,
            start_date,
            end_date,
            WeatherTypeCollection.WEATHER_HISTORICAL_DAILY,
            indicators,
        )

        assert {'weatherType', 'precipitation.cumulative', 'precipitation.probabilities', 'temperature.standard',
                'temperature.standardMax', 'Location'}.issubset(set(df.columns))
        assert df.index.name == "date"
        assert df["weatherType"].iloc[1] == "FORECAST_DAILY"

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_satellite_image_time_series(self, get_response):
        fake_get_tiff_zip_response =  mock_http_response_binary_content("GET", load_binary_data_from_zipfile("Refletance_map_mock.tiff.zip"))
        fake_image_time_series_response =  mock_http_response_text_content("GET", load_data_from_textfile(
           "satellite_image_time_series_landsat8_mock_http_response"))
        get_response.side_effect= [fake_image_time_series_response, fake_get_tiff_zip_response, fake_get_tiff_zip_response]
        start_date = dt.datetime.strptime("2022-05-01", "%Y-%m-%d")
        end_date = dt.datetime.strptime("2023-04-28", "%Y-%m-%d")
        dataset = self.client.get_satellite_image_time_series(
            POLYGON,
            start_date,
            end_date,
            collections=[SatelliteImageryCollection.SENTINEL_2, SatelliteImageryCollection.LANDSAT_8],
            indicators=["Reflectance"],
        )
        assert dict(dataset.dims) == {'band': 4, 'y': 80, 'x': 81, 'time': 2}


    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_agriquest_weather_block_data(self, get_response):
        get_response.return_value =  mock_http_response_text_content("POST", load_data_from_textfile(
           "agriquest_weather_data_mock_http_response"))
        start_date = "2022-05-01"
        end_date = "2023-04-28"
        dataset = self.client.get_agriquest_weather_block_data(
            start_date=start_date,
            end_date=end_date,
            block_code=AgriquestBlocks.FRA_DEPARTEMENTS,
            weather_type=AgriquestWeatherType.CUMULATIVE_PRECIPITATION
        )
        assert dataset.keys()[0] == "AMU"
        assert len(dataset["AMU"]) == 97

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_agriquest_ndvi_block_data(self, get_response):
        get_response.return_value =  mock_http_response_text_content("POST", load_data_from_textfile(
           "agriquest_ndvi_data_mock_http_response"))
        date = "2023-06-05"
        dataset = self.client.get_agriquest_ndvi_block_data(
            day_of_measure=date,
            commodity_code=AgriquestCommodityCode.ALL_VEGETATION,
            block_code=AgriquestBlocks.AMU_NORTH_AMERICA,
        )
        assert dataset.keys()[0] == "AMU"
        assert dataset.keys()[-1] == "NDVI"



import logging
import pandas as pd
from datetime import datetime
from urllib.parse import urljoin
from geosyspy.utils.constants import *
from geosyspy.utils.http_client import *
from typing import Optional


class AnalyticsFabricService:

    def __init__(self, base_url: str, http_client: HttpClient):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.logger = logging.getLogger(__name__)


    @staticmethod
    def build_timestamp_query_parameters(start_date: Optional[datetime] = None,
                                         end_date: Optional[datetime] = None):
        """ Build Timestamp parameter to provide in AF api calls
        Args:
            start_date : A datetime object representing the start date of the date interval the user wants to filter on.
            end_date : A datetime object representing the final date of the date interval the user wants to filter on.
        Returns:
            A string query params to include in api call
        """

        if start_date is None and end_date is None:
            return ""
        if start_date is not None and end_date is None:
            return f'&Timestamp=$gte:{start_date}'
        elif start_date is None and end_date is not None:
            return f'&Timestamp:$lte:{end_date}'
        else:
            return f'&Timestamp=$between:{start_date}|{end_date}'

    def create_schema_id(self, schema_id: str,
                         schema: dict):
        """Create a schema in Analytics Fabrics

        Args:
            schema_id: The schema id to create
            schema: Dict representing the schema {'property_name': 'property_type'}

        Returns:
            A http response object.
        """
        properties = []
        for prop_name, datatype in schema.items():
            prop = {
                "Name": prop_name,
                "Datatype": datatype,
                "UnitCategory": None,
                "IsPartOfKey": False,
                "IsOptional": False,
            }
            properties.append(prop)

        payload = {
            "Id": schema_id,
            "Properties": properties,
            "Metadata": {"OnAggregationCompleted": "Off"},
        }
        af_url: str = urljoin(
            self.base_url,
            GeosysApiEndpoints.ANALYTICS_FABRIC_SCHEMA_ENDPOINT.value,
        )
        response = self.http_client.post(af_url, payload)
        dict_response = response.json()
        if response.status_code == 201 :
            return response.content
        elif response.status_code == 400 and "This schema already exists." in str(dict_response["Errors"]["Body"]["Id"]):
            self.logger.info(f"The schema {schema_id} already exists.")
        else:
            self.logger.info(response.status_code)

    def get_metrics(self, season_field_id: str,
                    schema_id: str,
                    start_date: Optional[datetime] = None,
                    end_date: Optional[datetime] = None):
        """Returns metrics from Analytics Fabrics in a pandas dataframe.
        Filters on date:
        if start_date is None: <= end_date
        if end_date is None: >= start_date
        if start_date & end_date not None:  between start_date & end_date
        if start_date & end_date both None: no filter on dates


        Args:
            season_field_id : A string representing the seasonfield id
            start_date : A datetime object representing the start date of the date interval the user wants to filter on.
            end_date : A datetime object representing the final date of the date interval the user wants to filter on.
            schema_id : A string representing a schema existing in Analytics Fabrics

        Returns:
            df : A Pandas DataFrame containing several columns with metrics

        """
        self.logger.info("Calling APIs for metrics")
        if start_date is not None:
            start_date: str = start_date.strftime("%Y-%m-%d")
        if end_date is not None:
            end_date: str = end_date.strftime("%Y-%m-%d")

        timestamp_params = self.build_timestamp_query_parameters(start_date, end_date);
        parameters: str = f'?%24filter=Entity.TypedId==\'SeasonField:{season_field_id}\'' \
                          f'{timestamp_params}' \
                          f'&Schema.Id={schema_id}' \
                          f'&%24limit=None'

        af_url: str = urljoin(
            self.base_url,
            GeosysApiEndpoints.ANALYTICS_FABRIC_ENDPOINT.value + parameters,
        )
        response = self.http_client.get(af_url)

        if response.status_code == 200:
            df = pd.json_normalize(response.json())
            if df.empty:
                if start_date is not None and end_date is not None:
                    date_msg =f"between:{start_date} and {end_date} "
                elif start_date is None and end_date is not None:
                    date_msg =f"<= {end_date} "
                elif start_date is not None and end_date is None:
                    date_msg = f">= {start_date} "
                self.logger.info(f"No metrics found in Analytic Fabric with "
                             f"SchemaId: {schema_id}, "
                             f"SeasonField:{season_field_id} "
                             f"{date_msg} ")
                return df
            df.drop("Entity.TypedId", inplace=True, axis=1)
            df.rename(
                columns={"Timestamp": "date"},
                inplace=True,
            )
            df = df.sort_values(by="date")
            df.set_index("date", inplace=True)
            return df
        else:
            self.logger.error("Issue in get_metrics. Status Code: "+str(response.status_code) +
                              " Error:" + str(response.json()))

    def get_lastest_metrics(self, season_field_id: str,
                            schema_id: str):
        """Returns latest metrics from Analytics Fabrics in a pandas dataframe.

        Args:
            season_field_id : A string representing the seasonfield id
            schema_id : A string representing a schema existing in Analytics Fabrics

        Returns:
            df : A Pandas DataFrame containing several columns with metrics

        """
        self.logger.info("Calling APIs for Latest metrics")

        parameters: str = f'?%24filter=Entity.TypedId==\'SeasonField:{season_field_id}\'' \
                          f'&Schema.Id={schema_id}' \
                          f'&%24limit=1' \
                          f'&$sort=-Timestamp'

        af_url: str = urljoin(
            self.base_url,
            GeosysApiEndpoints.ANALYTICS_FABRIC_LATEST_ENDPOINT.value + parameters,
        )
        response = self.http_client.get(af_url)

        if response.status_code == 200:
            df = pd.json_normalize(response.json())
            if df.empty:
                self.logger.info(f"No Latest metrics found in Analytic Fabric with "
                             f"SchemaId: {schema_id}, "
                             f"SeasonField:{season_field_id} ")
                return df
            df.drop("Entity.TypedId", inplace=True, axis=1)
            df.rename(
                columns={"Timestamp": "date"},
                inplace=True,
            )
            df = df.sort_values(by="date")
            df.set_index("date", inplace=True)
            return df
        else:
            self.logger.error("Issue in get_latests_metrics. Status Code: "+str(response.status_code)
                              + " Error:" + str(response.json()))

    def push_metrics(self, season_field_id: str,
                     schema_id: str,
                     values: dict):
        """Push metrics in Analytics Fabrics

        Args:
            season_field_id : A string representing the seasonFieldId.
            schema_id : The schema on which to save
            values : Dict representing values to push

        Returns:
            A response object.
        """
        payload = []
        for value in values:
            prop = {
                "Entity": {
                    "TypedId": f"SeasonField:{season_field_id}@LEGACY_ID_NA"
                },
                "Schema": {"Id": schema_id, "Version": 1},
            }
            prop = dict(prop, **value)
            payload.append(prop)

        af_url: str = urljoin(
            self.base_url,
            GeosysApiEndpoints.ANALYTICS_FABRIC_ENDPOINT.value,
        )
        response = self.http_client.patch(af_url, payload)
        if response.status_code == 200:
            return response.status_code
        else:
            self.logger.error("Issue in push_metrics. Status Code: "+str(response.status_code)
                              + " Error:" + str(response.json()))



from urllib.parse import urljoin
import logging
from geosyspy.utils.http_client import *


class GisService:

    def __init__(self, base_url: str, http_client: HttpClient):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.logger = logging.getLogger(__name__)

    def get_municipio_id_from_geometry(self,
                                       geometry:str
                                       ):
        """
            method to call Gis Api to retrieve Municipio id from a geometry

                Args:
                     geometry (str): the geometry (WKT or GeosJson) to retrieve the municipio Id
                Returns:
                    the internal id of the municipio
        """

        payload = {
            "properties": ["id"],
            "features": [geometry]
        }
        parameters: str = f"/layerservices/api/v1/layers/BRAZIL_MUNICIPIOS/intersect"
        gis_url: str = urljoin(self.base_url, parameters)
        response = self.http_client.post(gis_url, payload)
        if response.status_code == 200:
            dict_response = response.json()

            try:
                # extract & return municipio id from response
                municipio_id = dict_response[0][0]["properties"]["id"]
                if isinstance(municipio_id, int):
                    return municipio_id
                else:
                    self.logger.warning("No municipio id found for this geometry")
                    return 0
            except:
                return 0

        else:
            self.logger.info(response.status_code)
            raise ValueError(
                f"No municipio id found for this geometry"
            )


from enum import Enum


class SatelliteImageryCollection(Enum):
    """
    Available imagery collections
    """
    MODIS = "MODIS"
    SENTINEL_2 = "SENTINEL_2"
    LANDSAT_8 = "LANDSAT_8"
    LANDSAT_9 = "LANDSAT_9"


class WeatherTypeCollection(Enum):
    """
    Available weather collections
    """
    WEATHER_FORECAST_DAILY = "FORECAST_DAILY"
    WEATHER_FORECAST_HOURLY = "FORECAST_HOURLY"
    WEATHER_HISTORICAL_DAILY = "HISTORICAL_DAILY"


class Env(Enum):
    """
    Environment to target (PROD, PREPROD)
    """
    PROD = "prod"
    PREPROD = "preprod"


class Region(Enum):
    """
    Region to target (NA)
    """
    NA = "na"


class Harvest(Enum):
    """
    Type of Harvest query used for Harvest analytics processor
    """
    HARVEST_IN_SEASON = "IN_SEASON"
    HARVEST_HISTORICAL = "HISTORICAL"

class Emergence(Enum):
    """
    Type of Emergence query used for Emergence analytics processor
    """
    EMERGENCE_IN_SEASON = "IN_SEASON"
    EMERGENCE_HISTORICAL = "HISTORICAL"
    EMERGENCE_DELAY = "DELAY"


class AgriquestCommodityCode(Enum):
    """
    Available AgriQuest Commodity values
    """
    ALL_VEGETATION = 33
    ALL_CROPS = 35


class AgriquestFranceBlockCode(Enum):
    """
    Available AgriQuest Block codes dedicated to France
    """
    FRA_CANTONS = 216
    FRA_COMMUNES = 135
    FRA_DEPARTEMENTS = 226


class AgriquestBlocks(Enum):
    """
    Available AgriQuest Block codes
    """
    FIRST_LEVEL = 129
    AMU_AUSTRALIA_LEVEL_1 = 205
    AMU_AUSTRALIA_LEVEL_2 = 206
    AMU_CHINA = 202
    AMU_EUROPE_RUSSIA = 197
    AMU_INDIA = 204
    AMU_MEXICO = 212
    AMU_NORTH_AMERICA = 207
    AMU_SOUTH_AFRICA = 213
    BM_REGIONS = 139
    CAR = 140
    COUNTY = 141
    FRA_CANTONS = 216
    FRA_COMMUNES = 135
    FRA_DEPARTEMENTS = 226
    MESOREGION = 131
    NORTH_AFRICA_AMU = 125
    RAION = 127
    SERBIA = 132
    SOUTH_AMERICA_MUNICIPIOS_2020 = 267
    SOUTH_AMERICA_AMU = 115
    SPAIN_COMARCAS = 136
    US_ASD = 130
    WESTERN_AFRICA_AMU = 122


class AgriquestWeatherType(Enum):
    """
    Available AgriQuest Weather types
    """
    CUMULATIVE_PRECIPITATION = "cumulative-precipitation"
    MIN_TEMPERATURE = "min-temperature"
    AVERAGE_TEMPERATURE = "average-temperature"
    MAX_TEMPERATURE = "max-temperature"
    MAX_WIND_SPEED = "max-wind-speed"
    RELATIVE_HUMIDITY = "relative-humidity"
    SNOW_DEPTH = "snow-depth"
    SOIL_MOISTURE = "soil-moisture"
    SOLAR_RADIATION = "solar-radiation"

class ZarcSoilType(Enum):
    """
    Available Soil Type values for analytics processor Zarc
    """
    SOIL_TYPE_1 = "1"
    SOIL_TYPE_2 = "2"
    SOIL_TYPE_3 = "3"
    NONE = None

class CropIdSeason(Enum):
    """
    Available season values  for analytics processor Zarc
    """
    SEASON_1="SEASON_1"
    SEASON_2="SEASON_2"


class ZarcCycleType(Enum):
    """
    Available season values  for analytics processor Zarc
    """
    CYCLE_TYPE_1 = "1"
    CYCLE_TYPE_2 = "2"
    CYCLE_TYPE_3 = "3"
    NONE = None


class GeosysApiEndpoints(Enum):
    """
    Available Geosys APIs Endpoints
    """
    MASTER_DATA_MANAGEMENT_ENDPOINT = "master-data-management/v6"
    VTS_ENDPOINT = "vegetation-time-series/v1/season-fields"
    VTS_BY_PIXEL_ENDPOINT = "vegetation-time-series/v1/season-fields/pixels"
    FLM_CATALOG_IMAGERY = "field-level-maps/v4/season-fields/{}/catalog-imagery"
    FLM_COVERAGE = "field-level-maps/v4/season-fields/{}/coverage"
    WEATHER_ENDPOINT = "Weather/v1/weather"
    ANALYTICS_FABRIC_ENDPOINT = "analytics/metrics"
    ANALYTICS_FABRIC_LATEST_ENDPOINT = "analytics/metrics-latest"
    ANALYTICS_FABRIC_SCHEMA_ENDPOINT = "analytics/schemas"
    AGRIQUEST_ENDPOINT = "agriquest/Geosys.Agriquest.CropMonitoring.WebApi/v0/api"
    # Analytics processor
    PROCESSOR_EVENTS_ENDPOINT = "analytics-pipeline/v1/processors/events"
    LAUNCH_PROCESSOR_ENDPOINT = "analytics-pipeline/v1/processors/{}/launch"


LR_SATELLITE_COLLECTION = [SatelliteImageryCollection.MODIS]
MR_SATELLITE_COLLECTION = [SatelliteImageryCollection.LANDSAT_8, SatelliteImageryCollection.LANDSAT_9,
                           SatelliteImageryCollection.SENTINEL_2]

PRIORITY_HEADERS = {"bulk": "Geosys_API_Bulk", "realtime": ""}
SEASON_FIELD_ID_REGEX = r"\sId:\s(\w+),"



import json
import logging
from urllib.parse import urljoin
from geosyspy.utils.constants import *
from geosyspy.services.service_constants import *
from geosyspy.utils.http_client import *
import retrying
import datetime


class AnalyticsProcessorService:

    def __init__(self, base_url: str, http_client: HttpClient):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.logger = logging.getLogger(__name__)

    @retrying.retry(wait_exponential_multiplier=1000, wait_exponential_max=10000, stop_max_attempt_number=50, retry_on_exception=lambda exc: isinstance(exc, KeyError))
    def wait_and_check_task_status(self, task_id: str):
        """Check task status until it is ended for a specific analytics processor run

        Args:
            task_id (str) : A string representing a task id

        Returns:
            task_status (str): the status of the task

        """

        events_endpoint: str = urljoin(self.base_url,
                                       GeosysApiEndpoints.PROCESSOR_EVENTS_ENDPOINT.value + "/" + task_id)

        while True:
            response = self.http_client.get(events_endpoint)
            if response.ok:
                dict_resp = json.loads(response.content)
                task_status = dict_resp["status"]
            else:
                self.logger.info(response.status_code)
                return "Failed"

            if task_status == "Ended":
                break  # the task is completed.
            elif task_status == "Running":
                self.logger.info("Retry -- Task still running")
                raise KeyError("Task still running")  # raise exception to retry
            else:
                raise Exception(f"Task Status: {task_status}, Content: {response.content}" )

        return task_status

    def get_s3_path_from_task_and_processor(self, task_id: str,
                                            processor_name: str):
        """Returns S3 path related to task_id

        Args:
            task_id : A string representing a task id
            processor_name: the processor name
        Returns:
            path : uri

        """

        events_endpoint: str = urljoin(self.base_url,
                                            GeosysApiEndpoints.PROCESSOR_EVENTS_ENDPOINT.value + "/" + task_id)

        response = self.http_client.get(events_endpoint)
        if response.ok:
            dict_resp = json.loads(response.content)
            customer_code: str = dict_resp["customerCode"].lower().replace("_", "-")
            user_id: str = dict_resp["userId"]
            task_id = dict_resp["taskId"]
            return f"s3://geosys-{customer_code}/{user_id}/{processor_name}/{task_id}"
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_mr_time_series_processor(self, polygon,
                                        start_date: str,
                                        end_date,
                                        list_sensors,
                                        denoiser: bool,
                                        smoother: str,
                                        eoc: bool,
                                        aggregation: str,
                                        index: str,
                                        raw_data: bool):
        """launch a MRTS analytics processor and get the task id in result

            Args:
                start_date : The start date of the time series
                end_date : The end date of the time series
                list_sensors : The Satellite Imagery Collection targeted
                denoiser : A boolean value indicating whether a denoising operation should be applied or not.
                smoother : The type or name of the smoothing technique or algorithm to be used.
                eoc : A boolean value indicating whether the "end of curve" detection should be performed.
                func : The type or name of the function to be applied to the data.
                index : The type or name of the index used for data manipulation or referencing
                raw_data : A boolean value indicating whether the data is in its raw/unprocessed form.
                polygon : A string representing a polygon.

            Returns:
                taskId (str)
        """

        if end_date is None:
            end_date = datetime.datetime.today().strftime("%Y-%m-%d")
        payload = {
            "parametersProfile": {
                "code": ProcessorConfiguration.MRTS.value['profile'],
                "version": 1
            },
            "parameters": {
                "start_date": start_date,
                "end_date": end_date,
                "sensors": list_sensors,
                "denoiser": denoiser,
                "smoother": smoother,
                "eoc": eoc,
                "aggregation": aggregation,
                "index": index,
                "raw_data": raw_data
            },
            "data": [
                {"wkt": polygon}
            ]
        }

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(
                                              ProcessorConfiguration.MRTS.value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_planted_area_processor(self,
                                      start_date: str,
                                      end_date: str,
                                      seasonfield_id: str):
        """launch a planted area analytics processor and get the task id in result

            Args:
                start_date (str) : the start date used for the request (format YYYY-MM-dd)
                end_date (str) : the end date used for the request (format YYYY-MM-dd)
                seasonfield_id (sfd) : seasonField geosys uniqueId

            Returns:
                taskId (str)
        """

        payload = {
            "parametersProfileCode": ProcessorConfiguration.PLANTED_AREA.value['profile'],
            "data":
                [
                    {
                        "id": seasonfield_id + "@ID",
                        "start_date": start_date,
                        "end_date": end_date
                    }
                ]
        }

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(ProcessorConfiguration.PLANTED_AREA.value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_harvest_processor(self,
                                 season_duration: int,
                                 season_start_day: int,
                                 season_start_month: int,
                                 crop: str,
                                 year: int,
                                 seasonfield_id: str,
                                 geometry: str,
                                 harvest_type: Harvest):
        """launch a harvest analytics processor and get the task id in result

            Args:
                season_duration (int): the duration of the season in days,
                season_start_day (int): the start day value (1 - 31),
                season_start_month (int): the start month value (1 - 12),
                crop (str): the geosys crop code,
                year (int): the year value,
                seasonfield_id (sfd) : seasonField geosys uniqueId
                geometry (str): the geometry to calcultate the analytic (WKT or GeoJSON),
                harvest_type (Harvest): the type of Harvest analytics (INSEASON/HISTORICAL)

            Returns:
                taskId (str)
        """

        # build payload for api call
        payload = {
            "parametersProfile": ProcessorConfiguration[harvest_type.name].value['profile'],
            "parameters": {
                "season_duration": season_duration,
                "season_start_day": season_start_day,
                "season_start_month": season_start_month
            },
            "data":
                [
                    {
                        "id": "SeasonField:" + seasonfield_id + "@ID",
                        "crop": crop,
                        "year": year,
                        "geom": geometry
                    }
                ]
        }

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(ProcessorConfiguration[harvest_type.name].value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_emergence_processor(self,
                                   season_duration: int,
                                   season_start_day: int,
                                   season_start_month: int,
                                   crop: str,
                                   year: int,
                                   seasonfield_id: str,
                                   geometry: str,
                                   emergence_type: Emergence):
        """launch an emergence analytics processor and get the task id in result

            Args:
                season_duration (int): the duration of the season in days,
                season_start_day (int): the start day value (1 - 31),
                season_start_month (int): the start month value (1 - 12),
                crop (str): the geosys crop code,
                year (int): the year value,
                seasonfield_id (sfd) : seasonField geosys uniqueId
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),
                emergence_type (Emergence): the type of Emergence analytics (INSEASON/HISTORICAL/DELAY)

            Returns:
                taskId (str)
        """

        # build payload for api call
        payload = {
            "parametersProfile": ProcessorConfiguration[emergence_type.name].value['profile'],
            "parameters": {
                "season_duration": season_duration,
                "season_start_day": season_start_day,
                "season_start_month": season_start_month
            },
            "data":
                [
                    {
                        "id": "SeasonField:" + seasonfield_id + "@ID",
                        "crop": crop,
                        "year": year,
                        "geom": geometry
                    }
                ]
        }

        if emergence_type == Emergence.EMERGENCE_DELAY:
            payload["parameters"]["emergence_delay"] = True

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format( ProcessorConfiguration[emergence_type.name].value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_potential_score_processor(self,
                                         season_duration: int,
                                         season_start_day: int,
                                         season_start_month: int,
                                         crop: str,
                                         end_date: str,
                                         sowing_date: str,
                                         nb_historical_years: int,
                                         seasonfield_id: str,
                                         geometry: str):
        """launch a potential score analytics processor and get the task id in result

            Args:
                season_duration (int): the duration of the season in days,
                season_start_day (int): the start day value (1 - 31),
                season_start_month (int): the start month value (1 - 12),
                crop (str): the geosys crop code,
                end_date (str): end date used to calculate potential score
                sowing_date (str): sowing date of the filed used to calculate potential score
                nb_historical_years (int): number of historical years data to calculate potential score
                seasonfield_id (sfd) : seasonField geosys uniqueId
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON)

            Returns:
                taskId (str)
        """

        # build payload for api call
        payload = {
            "parametersProfile":  ProcessorConfiguration.POTENTIAL_SCORE.value['profile'],
            "parameters": {
                "end_date" : end_date,
                "nb_historical_years" : nb_historical_years,
                "season_duration": season_duration,
                "season_start_day": season_start_day,
                "season_start_month": season_start_month
            },
            "data":
                [
                    {
                        "id": "SeasonField:" + seasonfield_id + "@ID",
                        "crop": crop,
                        "sowing_date": sowing_date,
                        "geom": geometry
                    }
                ]
        }

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(ProcessorConfiguration.POTENTIAL_SCORE.value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_brazil_in_season_crop_id_processor(self,
                                                  start_date: str,
                                                  end_date: str,
                                                  season: str,
                                                  seasonfield_id: str,
                                                  geometry: str):
        """launch a brazil-in-season-crop-id analytics processor and get the task id in result

            Args:
                start_date (str) : the start date used for the request (format YYYY-MM-dd)
                end_date (str) : the end date used for the request (format YYYY-MM-dd)

                season (str): the season name,
                seasonfield_id (sfd) : seasonField geosys uniqueId
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),

            Returns:
                taskId (str)
        """

        # build payload for api call
        payload = {
            "parametersProfile": ProcessorConfiguration.BRAZIL_IN_SEASON_CROP_ID.value['profile'],
            "parameters": {
                "start_date": start_date,
                "end_date": end_date,
                "season": season
            },
            "data":
                [
                    {
                        "id": "SeasonField:" + seasonfield_id + "@ID",
                        "geom": geometry
                    }
                ]
        }

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(ProcessorConfiguration.BRAZIL_IN_SEASON_CROP_ID.value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_greenness_processor(self,
                                   start_date: str,
                                   end_date: str,
                                   sowing_date: str,
                                   crop: str,
                                   seasonfield_id: str,
                                   geometry: str):
        """launch a greenness analytics processor and get the task id in result

            Args:
                start_date (str) : the start date used for the request (format YYYY-MM-dd)
                end_date (str) : the end date used for the request (format YYYY-MM-dd)
                sowing_date(str): sowing date of the field used to calculate potential score
                crop (str): the EDA crop code,
                seasonfield_id (sfd) : seasonField geosys uniqueId
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),

            Returns:
                taskId (str)
        """

        # build payload for api call
        payload = {
            "parametersProfile": ProcessorConfiguration.GREENNESS.value['profile'],
            "parameters": {
                "start_date": start_date,
                "end_date": end_date,
            },
            "data":
                [
                    {
                        "id": "SeasonField:" + seasonfield_id + "@ID",
                        "crop": crop,
                        "sowing_date": sowing_date,
                        "geom": geometry
                    }
                ]
        }

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(ProcessorConfiguration.GREENNESS.value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)


    def launch_harvest_readiness_processor(self,
                                   start_date: str,
                                   end_date: str,
                                   sowing_date: str,
                                   crop: str,
                                   seasonfield_id: str,
                                   geometry: str):
        """launch a harvest readiness analytics processor and get the task id in result

            Args:
                start_date (str) : the start date used for the request (format YYYY-MM-dd)
                end_date (str) : the end date used for the request (format YYYY-MM-dd)
                sowing_date(str): sowing date of the field used to calculate potential score
                crop (str): the EDA crop code,
                seasonfield_id (sfd) : seasonField geosys uniqueId
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),

            Returns:
                taskId (str)
        """

        # build payload for api call
        payload = {
            "parametersProfile": ProcessorConfiguration.HARVEST_READINESS.value['profile'],
            "parameters": {
                "start_date": start_date,
                "end_date": end_date,
            },
            "data":
                [
                    {
                        "id": "SeasonField:" + seasonfield_id + "@ID",
                        "crop": crop,
                        "sowing_date": sowing_date,
                        "geom": geometry
                    }
                ]
        }

        processor_endpoint: str = urljoin(self.base_url,
                                          GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(ProcessorConfiguration.HARVEST_READINESS.value['api_processor_path']))

        response = self.http_client.post(processor_endpoint, payload)

        if response.ok:
            task_id = json.loads(response.content)["taskId"]
            return task_id
        else:
            self.logger.info(response.status_code)
            raise ValueError(response.content)

    def launch_zarc_processor(self,
                              start_date_emergence: str,
                              end_date_emergence: str,
                              nb_days_sowing_emergence: int,
                              crop: str,
                              municipio: int,
                              soil_type: str,
                              cycle: str,
                              seasonfield_id:str):
            """launch a zarc analytics processor and get the task id in result

                Args:
                    start_date_emergence (str) : the emergence start date used for the request (format YYYY-MM-dd)
                    end_date_emergence (str) : the emergence end date used for the request (format YYYY-MM-dd)
                    nb_days_sowing_emergence (int): the number of days for sowing emergence
                    crop (str): the zarc crop code,
                    municipio (int): the municipio id,
                    soil_type (str): the zarc soil type,
                    cycle (str): the zarc cycle value,
                    seasonfield_id (sfd) : seasonField geosys uniqueId

                Returns:
                    taskId (str)
            """

            # build payload for api call
            payload = {
                "parametersProfile": ProcessorConfiguration.ZARC.value['profile'],
                "data":
                    [
                        {
                            "id": "SeasonField:" + seasonfield_id + "@ID",
                            "start_date_emergence": start_date_emergence,
                            "end_date_emergence": end_date_emergence,
                            "crop_zarc": crop,
                            "municipio_zarc": municipio,
                            "nb_days_sowing_emergence": nb_days_sowing_emergence,
                            "soil_type_zarc": soil_type,
                            "cycle_zarc": cycle
                        }
                    ]
            }

            processor_endpoint: str = urljoin(self.base_url,
                                              GeosysApiEndpoints.LAUNCH_PROCESSOR_ENDPOINT.value.format(
                                                  ProcessorConfiguration.ZARC.value['api_processor_path']))

            response = self.http_client.post(processor_endpoint, payload)

            if response.ok:
                task_id = json.loads(response.content)["taskId"]
                return task_id
            else:
                self.logger.info(response.status_code)
                raise ValueError(response.content)




import logging
import pandas as pd
from datetime import datetime
from urllib.parse import urljoin

from shapely import wkt

from geosyspy.utils.constants import *
from geosyspy.utils.http_client import *


class WeatherService:

    def __init__(self, base_url: str, http_client: HttpClient):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.logger = logging.getLogger(__name__)

    def get_weather(self, polygon: str,
                      start_date: datetime,
                      end_date: datetime,
                      weather_type: WeatherTypeCollection,
                      fields: [str]):
        """Returns the weather data as a pandas dataframe.

        Args:
            polygon : A string representing a polygon.
            start_date : A datetime object representing the start date of the date interval the user wants to filter on.
            end_date : A datetime object representing the final date of the date interval the user wants to filter on.
            weather_type : A list representing the weather type collection ["HISTORICAL_DAILY", "FORECAST_DAILY", "FORECAST_HOURLY"]
            fields : A list of strings representing the fields to select (eg: Precipitation, Temperature)

        Returns:
            The image's numpy array.

        """

        if weather_type not in WeatherTypeCollection:
            raise ValueError(f"weather_type should be either {[item.value for item in WeatherTypeCollection]}")
        weather_type = weather_type.value
        if "Date" not in fields:
            fields.append("Date")

        start_date: str = start_date.strftime("%Y-%m-%d")
        end_date: str = end_date.strftime("%Y-%m-%d")
        polygon_wkt = wkt.loads(polygon)
        weather_fields: str = ",".join(fields)
        parameters: str = f"?%24offset=0&%24limit=None&%24count=false&Location={polygon_wkt.centroid.wkt}&Date=%24between%3A{start_date}T00%3A00%3A00.0000000Z%7C{end_date}T00%3A00%3A00.0000000Z&Provider=GLOBAL1&WeatherType={weather_type}&$fields={weather_fields}"
        weather_url: str = urljoin(self.base_url, GeosysApiEndpoints.WEATHER_ENDPOINT.value + parameters)

        response = self.http_client.get(weather_url)

        if response.status_code == 200:
            df = pd.json_normalize(response.json())
            if df.empty:
                return df
            else:
                df.set_index("date", inplace=True)
                df["Location"] = polygon_wkt.centroid.wkt
                return df.sort_index()
        else:
            self.logger.error(response.status_code)
            raise ValueError(response.content)


import datetime as dt
import numpy as np
from unittest.mock import patch

from geosyspy.services.vegetation_time_series_service import VegetationTimeSeriesService
from geosyspy.utils.http_client import *
from tests.test_helper import *


class TestVegetationTimeSeriesService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                             "client_secret_123456",
                             "username_123",
                             "password_123",
                             "preprod",
                             "na")
    service = VegetationTimeSeriesService(base_url=url, http_client=http_client)

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_time_series_modis_ndvi(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "time_series_modis_ndvi_mock_http_response"))
        start_date = dt.datetime.strptime("2020-01-01", "%Y-%m-%d")
        end_date = dt.datetime.strptime("2020-01-07", "%Y-%m-%d")
        df = self.service.get_modis_time_series(
            "fakeSeasonFieldId", start_date, end_date, ["NDVI"]
        )

        assert df.index.name == "date"
        assert "value" in df.columns
        assert "index" in df.columns
        assert len(df.index) == 7
        date_range = list(map(lambda x: x.strftime("%Y-%m-%d"), df.index))
        assert {"2020-01-01", "2020-01-02", "2020-01-03", "2020-01-04", "2020-01-05", "2020-01-06",
                "2020-01-07"}.issubset(set(date_range))

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_satellite_image_time_series_modis_ndvi(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "satellite_image_time_series_modis_ndvi_mock_http_response"))
        start_date = dt.datetime.strptime("2020-01-01", "%Y-%m-%d")
        end_date = dt.datetime.strptime("2020-01-07", "%Y-%m-%d")

        df = self.service.get_time_series_by_pixel(
            "fakeSeasonFieldId", start_date, end_date, ["NDVI"]
        )
        assert df.index.name == "date"
        assert {"value", "index", "pixel.id"}.issubset(set(df.columns))
        assert np.all((df["index"].values == "NDVI"))
        assert len(df.index) == 14

        assert {"2020-01-01", "2020-01-02", "2020-01-03", "2020-01-04", "2020-01-05", "2020-01-06",
                "2020-01-07"}.issubset(set(df.index))

        assert {"mh11v4i225j4612", "mh11v4i226j4612"}.issubset(set(df["pixel.id"]))





import jwt
import base64
import json
from datetime import datetime, timezone
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
import logging


logger = logging.getLogger(__name__)


def check_token_validity(token: str, certificate_key: str, algorithms=['RS256']) -> bool:
    """
    Check the validity of a JWT token.

    Args:
        token (str): The JWT token to check.
        certificate_key (str): The certificate key in PEM format.
        algorithms (list, optional): The encryption algorithms to use.
            Default is ['RS256'].

    Returns:
        bool: True if the token is valid, False otherwise.
    """
    try:
        cert_bytes = certificate_key.encode()
        cert = x509.load_pem_x509_certificate(cert_bytes, default_backend())
        public_key = cert.public_key()

        # extract public key in PEM format
        public_key = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        ).decode()
        aud = __get_audience_from_token(token)
        decoded_token = jwt.decode(token, public_key, algorithms=algorithms, audience=aud)
        expiration_timestamp = decoded_token['exp']
        expiration_datetime = datetime.fromtimestamp(expiration_timestamp, tz=timezone.utc)

        if expiration_datetime > datetime.now(timezone.utc):
            return True
        else:
            return False
    except jwt.ExpiredSignatureError:
        logger.error("Expired Token")
        return False
    except jwt.InvalidTokenError as e:
        logger.error("Invalid Token." + str(e))
        return False
    except Exception as e:
        logger.error("Invalid Token." + str(e))
        return False


def __get_audience_from_token(token: str) -> str:
    """
    Get the audience from a JWT token.

    Args:
        token (str): The JWT token.

    Returns:
        str: The audience value.
    """
    # parse the token (header, payload, signature)
    header, payload, signature = token.split('.')

    # add missing data
    padding = '=' * (4 - len(payload) % 4)
    payload += padding

    # payload decoding & serialization
    decoded_payload = base64.urlsafe_b64decode(payload.encode('utf-8'))
    payload_json = decoded_payload.decode('utf-8')

    payload_dict = json.loads(payload_json)
    audience = payload_dict.get('aud')
    return audience



import json
import logging
import pandas as pd
from datetime import datetime
from urllib.parse import urljoin
from geosyspy.utils.constants import *
from geosyspy.utils.http_client import *
from requests import HTTPError


class MapProductService:

    def __init__(self, base_url: str, http_client: HttpClient, priority_queue: str):
        self.base_url: str = base_url
        self.http_client: HttpClient = http_client
        self.priority_queue: str = priority_queue
        self.logger = logging.getLogger(__name__)

    def get_satellite_coverage(self, season_field_id: str,
                               start_date: datetime,
                               end_date: datetime,
                               indicator,
                               sensors_collection: list[SatelliteImageryCollection] = [
                                   SatelliteImageryCollection.SENTINEL_2,
                                   SatelliteImageryCollection.LANDSAT_8]
                               ):

        self.logger.info("Calling APIs for coverage")
        start_date: str = start_date.strftime("%Y-%m-%d")
        end_date: str = end_date.strftime("%Y-%m-%d")
        sensors: list[str] = [elem.value for elem in sensors_collection]

        if indicator == "" or indicator.upper() == "REFLECTANCE" or indicator.upper() == "NDVI":
            mapType = "INSEASON_NDVI"
        else:
            mapType = f"INSEASON_{indicator.upper()}"

        parameters = f"?maps.type={mapType}&Image.Sensor=$in:{'|'.join(sensors)}&CoverageType=CLEAR&$limit=9999&$filter=Image.Date >= '{start_date}' and Image.Date <= '{end_date}'"

        flm_url: str = urljoin(
            self.base_url,
            GeosysApiEndpoints.FLM_CATALOG_IMAGERY.value.format(season_field_id) + parameters,
        )
        response = self.http_client.get(
            flm_url,
            {"X-Geosys-Task-Code": PRIORITY_HEADERS[self.priority_queue]},
        )

        if response.status_code == 200:
            df = pd.json_normalize(response.json())
            if df.empty:
                return df
            else:
                return df[
                    [
                        "coverageType",
                        "maps",
                        "image.id",
                        "image.availableBands",
                        "image.sensor",
                        "image.soilMaterial",
                        "image.spatialResolution",
                        "image.weather",
                        "image.date",
                        "seasonField.id",
                    ]
                ]
        else:
            self.logger.info(response.status_code)

    def get_zipped_tiff(self, field_id: str,
                          image_id: str,
                          indicator: str = ""):
        parameters = f"/{image_id}/reflectance-map/TOC/image.tiff.zip"

        if indicator != "" and indicator.upper() != "REFLECTANCE":
            parameters = f"/{image_id}/base-reference-map/INSEASON_{indicator.upper()}/image.tiff.zip?resolution=Sensor"

        download_tiff_url: str = urljoin(
            self.base_url, GeosysApiEndpoints.FLM_COVERAGE.value.format(field_id) + parameters
        )

        response_zipped_tiff = self.http_client.get(
            download_tiff_url,
            {"X-Geosys-Task-Code": PRIORITY_HEADERS[self.priority_queue]},
        )
        if response_zipped_tiff.status_code != 200:
            raise HTTPError("Unable to download tiff.zip file. Server error: " + str(response_zipped_tiff.status_code))
        return response_zipped_tiff


import datetime
import json
from unittest.mock import patch

from geosyspy.services.analytics_fabric_service import AnalyticsFabricService
from geosyspy.utils.http_client import *
from tests.test_helper import *

geometry = "POLYGON((-91.17523978603823 40.29787117039518,-91.17577285022956 40.29199489606421,-91.167613719932 40.29199489606421,-91.1673028670095 40.29867040193312,-91.17523978603823 40.29787117039518))"


class TestAnalyticsFabricService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")
    service = AnalyticsFabricService(base_url=url, http_client=http_client)

    def test_get_build_timestamp_query_parameters(self):
        start_date = datetime.datetime.now()
        end_date = start_date + datetime.timedelta(days=7, hours=3, minutes=30)

        timestamp = self.service.build_timestamp_query_parameters(end_date=end_date)
        assert timestamp == f'&Timestamp:$lte:{end_date}'

        timestamp = self.service.build_timestamp_query_parameters(start_date=start_date)
        assert timestamp == f'&Timestamp=$gte:{start_date}'

        timestamp = self.service.build_timestamp_query_parameters(start_date=start_date, end_date=end_date)
        assert timestamp == f'&Timestamp=$between:{start_date}|{end_date}'

        timestamp = self.service.build_timestamp_query_parameters()
        assert timestamp == ''

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_metrics(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "metrics_harvest_mock_http_response"))
        start_date = datetime.datetime.now()
        end_date = start_date + datetime.timedelta(days=7, hours=3, minutes=30)

        metrics = self.service.get_metrics(start_date=start_date, end_date=end_date, season_field_id= 'seasonfieldFakeId', schema_id='HISTORICAL_HARVEST')

        assert metrics['Schema.Id'][0] == 'HISTORICAL_HARVEST'
        assert metrics['Values.harvest_year_1'][0] == '2020-10-02'

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_latest_metrics(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "metrics_harvest_mock_http_response"))

        metrics = self.service.get_lastest_metrics(season_field_id= 'seasonfieldFakeId', schema_id='HISTORICAL_HARVEST')

        assert metrics['Schema.Id'][0] == 'HISTORICAL_HARVEST'
        assert metrics['Values.harvest_year_1'][0] == '2020-10-02'

    @patch('geosyspy.utils.http_client.HttpClient.patch')
    def test_push_metrics(self, patch_response):
        patch_response.return_value = mock_http_response_text_content("PATCH", load_data_from_textfile(
            "metrics_harvest_mock_http_response"))
        data = [{
            "Timestamp": "2022-01-01",
            "Values": {
                "NDVI": 0.5
            }
        }]

        result = self.service.push_metrics(season_field_id='seasonfieldFakeId', schema_id='HISTORICAL_HARVEST', values=data)

        assert result == 200

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_create_schema(self, get_response):
        get_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "create_schema_mock_http_response"), status_code=201)
        schema ={
            "Timestamp": '2021-01-01',
            "Values": {
                "NDVI": 0.5
            }
        }
        response = self.service.create_schema_id(schema_id='NEW_SCHEMA', schema = schema)
        data = json.loads(response.decode('utf-8'))
        assert data['Id'] == "NEW_SCHEMA"




from datetime import datetime
from unittest.mock import patch
from geosyspy import Geosys
from dotenv import load_dotenv
import datetime as dt
import numpy as np

from geosyspy.services.gis_service import GisService
from geosyspy.utils.constants import *
from geosyspy.utils.http_client import *
from tests.test_helper import *

geometry = "POLYGON((-91.17523978603823 40.29787117039518,-91.17577285022956 40.29199489606421,-91.167613719932 40.29199489606421,-91.1673028670095 40.29867040193312,-91.17523978603823 40.29787117039518))"


class TestGisService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")
    service = GisService(base_url=url, http_client=http_client)

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_municipio_id_from_geometry(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "gis_layer_municipio_data_mock_http_response"))

        municipio_id = self.service.get_municipio_id_from_geometry(geometry=geometry)
        assert municipio_id == 121935


from unittest.mock import patch

from geosyspy.services.master_data_management_service import MasterDataManagementService
from geosyspy.utils.http_client import *
from tests.test_helper import *

geometry = "POLYGON((-91.17523978603823 40.29787117039518,-91.17577285022956 40.29199489606421,-91.167613719932 40.29199489606421,-91.1673028670095 40.29867040193312,-91.17523978603823 40.29787117039518))"


class TestMasterDataManagementService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")
    service = MasterDataManagementService(base_url=url, http_client=http_client)

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_create_season_field_id(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "master_data_management_post_extract_id_mock_http_response"))

        response = self.service.create_season_field_id(polygon=geometry)
        assert response.status_code == 200

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_extract_season_field_id(self, post_response):
        post_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "master_data_management_post_extract_id_mock_http_response"),status_code=201)

        response = self.service.extract_season_field_id(polygon=geometry)
        assert response == "ajqxm3v"

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_extract_season_field_id(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "master_data_management_get_unique_id_mock_http_response"), status_code=201)

        response = self.service.get_season_field_unique_id(season_field_id="fakeSeasonFieldId")
        assert response == "4XcGhZvA1OjpO3gUwYM61e"


import datetime
import json
from unittest.mock import patch

from geosyspy.services.agriquest_service import AgriquestService
from geosyspy.utils.http_client import *
from tests.test_helper import *
from geosyspy.utils.constants import *

geometry = "POLYGON((-91.17523978603823 40.29787117039518,-91.17577285022956 40.29199489606421,-91.167613719932 40.29199489606421,-91.1673028670095 40.29867040193312,-91.17523978603823 40.29787117039518))"


class TestAgriquestService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                             "client_secret_123456",
                             "username_123",
                             "password_123",
                             "preprod",
                             "na")
    service = AgriquestService(base_url=url, http_client=http_client)

    def test_is_block_for_france(self):
        is_france_block = self.service.is_block_for_france(block_code=AgriquestBlocks.FRA_DEPARTEMENTS)
        assert is_france_block == True

        is_france_block = self.service.is_block_for_france(block_code=AgriquestBlocks.AMU_NORTH_AMERICA)
        assert is_france_block == False

    def test_weather_indicators_builder(self):
        start_date = datetime.datetime.now() - datetime.timedelta(days=1)
        end_date = start_date + datetime.timedelta(days=7, hours=3, minutes=30)

        indicators = self.service.weather_indicators_builder(start_date=start_date.date(), end_date = end_date.date(), isFrance = True)
        assert set(indicators) == set([3,4,5])

        indicators = self.service.weather_indicators_builder(start_date=start_date.date(), end_date=end_date.date(),
                                                             isFrance=False)
        assert set(indicators) == set([2, 4, 5])

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_agriquest_weather_block_data(self, get_response):
        get_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "agriquest_weather_data_mock_http_response"))
        start_date = "2022-05-01"
        end_date = "2023-04-28"

        dataset = self.service.get_agriquest_block_weather_data(
            start_date=start_date,
            end_date=end_date,
            block_code=AgriquestBlocks.FRA_DEPARTEMENTS,
            weather_type=AgriquestWeatherType.CUMULATIVE_PRECIPITATION,
            indicator_list=[3,4,5]
        )
        assert dataset.keys()[0] == "AMU"
        assert len(dataset["AMU"]) == 97

    @patch('geosyspy.utils.http_client.HttpClient.post')
    def test_get_agriquest_ndvi_block_data(self, get_response):
        get_response.return_value = mock_http_response_text_content("POST", load_data_from_textfile(
            "agriquest_ndvi_data_mock_http_response"))
        date = "2023-06-05"
        dataset = self.service.get_agriquest_block_ndvi_data(
            date=date,
            commodity=AgriquestCommodityCode.ALL_VEGETATION,
            block_code=AgriquestBlocks.AMU_NORTH_AMERICA,
            indicator_list=[1]
        )
        assert dataset.keys()[0] == "AMU"
        assert dataset.keys()[-1] == "NDVI"



# geosyspy/logging_config.py

import logging


def configure_logging(level=logging.INFO, handlers=None):
    """
    Configure logs for geosyspy.
    Args:
        level (int): log level
        handlers ([logging.handler]): logger handlers
    """
    logger = logging.getLogger('geosyspy')
    if handlers is None:
        handlers = [logging.StreamHandler()]  # By default, use a console handler

    for handler in handlers:
        handler.setLevel(level)
        handler.setFormatter(logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        ))
        logger.addHandler(handler)




from unittest.mock import patch
import datetime
from geosyspy.services.weather_service import WeatherService
from geosyspy.utils.http_client import *
from tests.test_helper import *
from geosyspy.utils.constants import *

geometry = "POLYGON((-91.17523978603823 40.29787117039518,-91.17577285022956 40.29199489606421,-91.167613719932 40.29199489606421,-91.1673028670095 40.29867040193312,-91.17523978603823 40.29787117039518))"

class TestWeatherService:
    url = "https://testurl.com"
    http_client = HttpClient("client_id_123",
                        "client_secret_123456",
                        "username_123",
                        "password_123",
                        "preprod",
                        "na")
    service = WeatherService(base_url=url, http_client=http_client)

    @patch('geosyspy.utils.http_client.HttpClient.get')
    def test_get_weather(self, get_response):
        get_response.return_value = mock_http_response_text_content("GET", load_data_from_textfile(
            "weather_data_mock_http_response"))
        start_date = datetime.datetime.now()
        end_date = start_date + datetime.timedelta(days=7, hours=3, minutes=30)

        data = self.service.get_weather(polygon=geometry, start_date=start_date, end_date=end_date,
                                        weather_type=WeatherTypeCollection.WEATHER_FORECAST_DAILY,
                                        fields=['precipitation','temperature'])

        assert data.index.__len__() == 6
        assert data['precipitation.cumulative'][0] == 0.22834645669291338
        assert data['temperature.standard'][0] == 71.47399998282076





from oauthlib.oauth2 import LegacyApplicationClient
from requests_oauthlib import OAuth2Session
from . import geosys_platform_urls
import logging


class Oauth2Api:
    def __init__(
            self,
            client_id: str,
            client_secret: str,
            username: str,
            password: str,
            enum_env: str,
            enum_region: str,
            bearer_token: str = None
    ):
        """Initializes a Geosys instance with the required credentials
        to connect to the GEOSYS API.
        """
        self.logger = logging.getLogger(__name__)
        self.client_id = client_id
        self.server_url = geosys_platform_urls.IDENTITY_URLS[enum_region][enum_env]
        self.client_secret = client_secret
        self.token = None
        self.username = username
        self.password = password

        if bearer_token:
            self.token = {"access_token": bearer_token}
        else:
            self.__authenticate()

    def __authenticate(self):
        """Authenticates the http_client to the API.

        This method connects the user to the API which generates a token that
        will be valid for one hour. A refresh token is also generated, which
        makes it possible for the http methods wrappers to get a new token
        once the previous one is no more valid through the renew_access_token
        decorator. This method is only run once when a Geosys object is instantiated.

        """

        try:
            oauth = OAuth2Session(
                client=LegacyApplicationClient(client_id=self.client_id)
            )
            self.token = oauth.fetch_token(
                token_url=self.server_url,
                username=self.username,
                password=self.password,
                client_id=self.client_id,
                client_secret=self.client_secret,
            )
            self.token["refresh_token"] = oauth.cookies["refresh_token"]
            self.logger.info("Authenticated")
        except Exception as e:
            logging.error(e)

    def get_refresh_token(self):
        """Fetches a new token."""
        client = OAuth2Session(self.client_id, token=self.token)
        return client.refresh_token(
            self.server_url,
            client_id=self.client_id,
            client_secret=self.client_secret,
        )



"""Setup script for geosyspy"""

# Standard library imports
import pathlib

# Third party imports
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).resolve().parent

# The text of the README file is used as a description
README = (HERE / "README.md").read_text()

VERSION = (HERE / "VERSION.txt").read_text()

# This call to setup() does all the work
setup(
    name="geosyspy",
    version=VERSION,
    description="Easy-to-use python wrapper for Geosys APIs (time series, imagery products)",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Geosys",
    packages=find_packages(),
    include_package_data=True,
    data_files=[('', ['VERSION.txt'])],
    install_requires=["requests", "requests-oauthlib", "oauthlib", "scipy", "pandas", "shapely", "rasterio", "xarray", "retrying"]
)


from unittest.mock import patch, call
from geosyspy.utils.oauth2_client import Oauth2Api


def get_oauth():
    return Oauth2Api("client_id_123",
                     "client_secret_123456",
                     "username_123",
                     "password_123",
                     "preprod",
                     "na")


def test_oauth2_should_initialize():
    oauth = get_oauth()
    assert oauth.client_id == "client_id_123"
    assert oauth.client_secret == "client_secret_123456"
    assert oauth.username == "username_123"
    assert oauth.password == "password_123"
    assert oauth.server_url != ""
    assert oauth.token is None

@patch("geosyspy.utils.oauth2_client.OAuth2Session")
def test_oauth2_get_token_should_work(OAuth2Session):
    OAuth2Session.return_value = OAuth2Session
    oauth = get_oauth()
    OAuth2Session.refresh_token.return_value = "628x9x0xx447xx4x421x517x4x474x33x2065x4x1xx523xxxxx6x7x20"
    oauth.get_refresh_token()
    assert OAuth2Session.refresh_token.call_count == 1
    OAuth2Session.refresh_token.assert_has_calls(
        [
            call(
                oauth.server_url,
                client_id=oauth.client_id,
                client_secret=oauth.client_secret
            )
        ]
    )







import io
import zipfile
from pathlib import Path

import numpy as np
import rasterio
import xarray as xr
from rasterio.io import MemoryFile

from geosyspy import image_reference
from geosyspy.services.agriquest_service import *
from geosyspy.services.analytics_fabric_service import *
from geosyspy.services.analytics_processor_service import *
from geosyspy.services.gis_service import *
from geosyspy.services.map_product_service import *
from geosyspy.services.master_data_management_service import *
from geosyspy.services.vegetation_time_series_service import *
from geosyspy.services.weather_service import *
from geosyspy.utils.geosys_platform_urls import *


class Geosys:
    """Geosys is the main client class to access all the Geosys APIs capabilities.

    `client = Geosys(api_client_id, api_client_secret, api_username, api_password, env, region)`

    Parameters:
        enum_env: 'Env.PROD' or 'Env.PREPROD'
        enum_region: 'Region.NA'
        priority_queue: 'realtime' or 'bulk'
    """

    def __init__(self,
                 client_id: str = None,
                 client_secret: str = None,
                 username: str = None,
                 password: str = None,
                 enum_env: Env = Env.PROD,
                 enum_region: Region = Region.NA,
                 priority_queue: str = "realtime",
                 bearer_token: str = None
                 ):
        self.logger = logging.getLogger(__name__)
        self.region: str = enum_region.value
        self.env: str = enum_env.value
        self.base_url: str = GEOSYS_API_URLS[enum_region.value][enum_env.value]
        self.gis_url: str = GIS_API_URLS[enum_region.value][enum_env.value]
        self.priority_queue: str = priority_queue
        self.http_client: HttpClient = HttpClient(client_id, client_secret, username, password, enum_env.value,
                                                  enum_region.value, bearer_token)
        self.__master_data_management_service = MasterDataManagementService(self.base_url, self.http_client)
        self.__analytics_fabric_service = AnalyticsFabricService(self.base_url, self.http_client)
        self.__analytics_processor_service = AnalyticsProcessorService(self.base_url, self.http_client)
        self.__agriquest_service = AgriquestService(self.base_url, self.http_client)
        self.__weather_service = WeatherService(self.base_url, self.http_client)
        self.__gis_service = GisService(self.gis_url, self.http_client)
        self.__vts_service = VegetationTimeSeriesService(self.base_url, self.http_client)
        self.__map_product_service = MapProductService(self.base_url, self.http_client, self.priority_queue)

    def get_time_series(self, polygon: str,
                        start_date: datetime,
                        end_date: datetime,
                        collection: enumerate,
                        indicators: [str]) -> pd.DataFrame:
        """Retrieve a time series of the indicator for the aggregated polygon on the collection targeted.

        Args:
            polygon : The polygon
            start_date : The start date of the time series
            end_date : The end date of the time series
            collection : The collection targeted
            indicators : The indicators to retrieve on the collection

        Returns:
            (dataframe): A pandas dataframe for the time series

        Raises:
            ValueError: The collection doesn't exist
        """
        if collection in WeatherTypeCollection:
            return self.__weather_service.get_weather(
                polygon,
                start_date,
                end_date,
                collection,
                indicators,
            )
        elif collection in LR_SATELLITE_COLLECTION:
            # extract seasonfield id from geometry
            season_field_id: str = self.__master_data_management_service.extract_season_field_id(polygon)
            return self.__vts_service.get_modis_time_series(
                season_field_id, start_date, end_date, indicators[0]
            )
        else:
            raise ValueError(f"{collection} collection doesn't exist")

    def get_satellite_image_time_series(self, polygon: str,
                                        start_date: datetime,
                                        end_date: datetime,
                                        collections: list[SatelliteImageryCollection],
                                        indicators: [str]
                                        ):
        """Retrieve a pixel-by-pixel time series of the indicator on the collection targeted.

        Args:
            polygon : The polygon
            start_date : The start date of the time series
            end_date : The end date of the time series
            collections : The Satellite Imagery Collection targeted
            indicators : The indicators to retrieve on the collections

        Returns:
            ('dataframe or xarray'): Either a pandas dataframe or a xarray for the time series
        """

        if not collections:
            raise ValueError(
                "The argument collections is empty. It must be a list of SatelliteImageryCollection objects"
            )
        elif all([isinstance(elem, SatelliteImageryCollection) for elem in collections]):
            # extract seasonfield id from geometry
            season_field_id: str = self.__master_data_management_service.extract_season_field_id(polygon)

            if set(collections).issubset(set(LR_SATELLITE_COLLECTION)):
                return self.__vts_service.get_time_series_by_pixel(
                    season_field_id, start_date, end_date, indicators[0]
                )
            elif set(collections).issubset(set(MR_SATELLITE_COLLECTION)):
                return self.__get_images_as_dataset(
                    season_field_id, start_date, end_date, collections, indicators[0]
                )
        else:
            raise TypeError(
                f"Argument collections must be a list of SatelliteImageryCollection objects"
            )

    def get_satellite_coverage_image_references(self, polygon: str,
                                                start_date: datetime,
                                                end_date: datetime,
                                                collections: list[SatelliteImageryCollection] = [
                                                    SatelliteImageryCollection.SENTINEL_2,
                                                    SatelliteImageryCollection.LANDSAT_8]
                                                ) -> tuple:
        """Retrieves a list of images that covers a polygon on a specific date range.
        The return is a tuple: a dataframe with all the images covering the polygon, and
                    a dictionary images_references. Key= a tuple (image_date, image_sensor).
                    Value = an object image_reference, to use with the method `download_image()`

        Args:
            polygon: The polygon
            start_date: The start date of the time series
            end_date: The end date of the time series
            collections: The sensors to check the coverage on

        Returns:
            (tuple): images list and image references for downloading
        """
        # extract seasonfield id from geometry
        season_field_id: str = self.__master_data_management_service.extract_season_field_id(polygon)

        df = self.__map_product_service.get_satellite_coverage(season_field_id, start_date, end_date, "", collections)
        images_references = {}
        if df is not None:
            for i, image in df.iterrows():
                images_references[
                    (image["image.date"], image["image.sensor"])
                ] = image_reference.ImageReference(
                    image["image.id"],
                    image["image.date"],
                    image["image.sensor"],
                    image["seasonField.id"],
                )

        return df, images_references

    def download_image(self, image_reference,
                       path: str = ""):
        """Downloads a satellite image locally

        Args:
            image_reference (ImageReference): An ImageReference object representing the image to download
            path (str): the path to download the image to
        """

        response_zipped_tiff = self.__map_product_service.get_zipped_tiff(
            image_reference.season_field_id, image_reference.image_id
        )
        if path == "":
            path = Path.cwd() / f"image_{image_reference.image_id}_tiff.zip"
        with open(path, "wb") as f:
            self.logger.info(f"writing to {path}")
            f.write(response_zipped_tiff.content)

    def __get_images_as_dataset(self, polygon: str,
                                start_date: datetime,
                                end_date: datetime,
                                collections: list[SatelliteImageryCollection],
                                indicator: str) -> 'np.ndarray[Any , np.dtype[np.float64]]':
        """Returns all the 'sensors_list' images covering 'polygon' between
        'start_date' and 'end_date' as a xarray dataset.

        Args:
            polygon : A string representing the polygon that the images will be covering.
            start_date : The date from which the method will start looking for images.
            end_date : The date at which the method will stop looking images.
            collections : A list of Satellite Imagery Collection.
            indicator : A string representing the indicator whose time series the user wants.

        Returns:
            The image's numpy array.

        """

        def get_coordinates_by_pixel(raster):
            """Returns the coordinates in meters in the raster's CRS
            from its pixels' grid coordinates."""

            img = raster.read()
            band1 = img[0]
            height = band1.shape[0]
            width = band1.shape[1]
            cols, rows = np.meshgrid(np.arange(width), np.arange(height))
            xs, ys = rasterio.transform.xy(raster.transform, rows, cols)
            lons = np.array(xs)
            lats = np.array(ys)
            lst_lats = [lat[0] for lat in lats]
            lst_lons = list(lons[0])
            return {"y": lst_lats, "x": lst_lons}

        # Selects the covering images in the provided date range
        # and sorts them by resolution, from the highest to the lowest.
        # Keeps only the first image if two are found on the same date.
        df_coverage = self.__map_product_service.get_satellite_coverage(
            polygon, start_date, end_date, indicator, collections
        )

        # Return empty dataset if no coverage on the polygon between start_date, end_date
        if df_coverage.empty:
            return xr.Dataset()

        df_coverage["image.date"] = pd.to_datetime(
            df_coverage["image.date"], infer_datetime_format=True
        )
        df_coverage = df_coverage.sort_values(
            by=["image.spatialResolution", "image.date"], ascending=[True, True]
        ).drop_duplicates(subset="image.date", keep="first")

        # Creates a dictionary that contains a zip archive containing the tif file
        # for each image id and some additional data (bands, sensor...)
        dict_archives = {}
        for i, row in df_coverage.iterrows():
            if indicator.upper() != "REFLECTANCE":
                bands = [indicator]
            else:
                bands = row["image.availableBands"]
            dict_archives[row["image.id"]] = {
                "byte_archive": self.__map_product_service.get_zipped_tiff(
                    row["seasonField.id"], row["image.id"], indicator
                ).content,
                "bands": bands,
                "date": row["image.date"],
                "sensor": row["image.sensor"],
            }

        # Extracts the tif files from  the zip archives in memory
        # and transforms them into a list of xarray DataArrays.
        # A list of all the raster's crs is also created in order
        # to merge this data in the final xarray Dataset later on.
        list_xarr = []
        list_crs = []
        first_img_id = df_coverage.iloc[0]["image.id"]
        for img_id, dict_data in dict_archives.items():
            with zipfile.ZipFile(io.BytesIO(dict_data["byte_archive"]), "r") as archive:
                images_in_bytes = [archive.read(file) for file in archive.namelist() if file.endswith('.tif')]
                for image in images_in_bytes:
                    with MemoryFile(image) as memfile:
                        with memfile.open() as raster:
                            dict_coords = get_coordinates_by_pixel(raster)
                            xarr = xr.DataArray(
                                raster.read(masked=True),
                                dims=["band", "y", "x"],
                                coords={
                                    "band": dict_data["bands"],
                                    "y": dict_coords["y"],
                                    "x": dict_coords["x"],
                                    "time": dict_data["date"],
                                },
                            )

                            if img_id == first_img_id:
                                len_y = len(dict_coords["y"])
                                len_x = len(dict_coords["x"])
                                print(
                                    f"The highest resolution's image grid size is {(len_x, len_y)}"
                                )
                            else:
                                self.logger.info(
                                    f"interpolating {img_id} to {first_img_id}'s grid"
                                )
                                xarr = xarr.interp(
                                    x=list_xarr[0].coords["x"].data,
                                    y=list_xarr[0].coords["y"].data,
                                    method="linear",
                                )
                            list_xarr.append(xarr)
                            list_crs.append(raster.crs.to_string())

        # Adds the img's raster's crs to the initial dataframe
        df_coverage["crs"] = list_crs

        # Concatenates all the DataArrays in list_xarr in order
        # to create one final DataArray with an additional dimension
        # 'time'. This final DataArray is then transformed into
        # a xarray Dataset containing one data variable "reflectance".

        final_xarr = xr.concat(list_xarr, "time")
        dataset = xr.Dataset(data_vars={indicator.lower(): final_xarr})

        # Adds additional metadata to the dataset.
        dataset = dataset.assign_coords(
            **{
                k: ("time", np.array(v))
                for k, v in df_coverage[
                    [
                        "image.id",
                        "image.sensor",
                        "image.soilMaterial",
                        "image.spatialResolution",
                        "image.weather",
                        "crs",
                    ]
                ]
                .to_dict(orient="list")
                .items()
            }
        )
        return dataset

    ###########################################
    #           ANALYTICS FABRIC              #
    ###########################################

    def create_schema_id(self, schema_id: str,
                         schema: dict):
        """Create a schema in Analytics Fabrics

        Args:
            schema_id: The schema id to create
            schema: Dict representing the schema {'property_name': 'property_type'}

        Returns:
            A http response object.
        """
        return self.__analytics_fabric_service.create_schema_id(schema_id=schema_id, schema=schema)

    def get_metrics(self, polygon: str,
                    schema_id: str,
                    start_date: datetime,
                    end_date: datetime):
        """Returns metrics from Analytics Fabrics in a pandas dataframe.

        Args:
            polygon : A string representing a polygon.
            start_date : A datetime object representing the start date of the date interval the user wants to filter on.
            end_date : A datetime object representing the final date of the date interval the user wants to filter on.
            schema_id : A string representing a schema existing in Analytics Fabrics

        Returns:
            df : A Pandas DataFrame containing severals columns with metrics

        """

        season_field_id: str = self.__master_data_management_service.extract_season_field_id(polygon)
        season_field_unique_id: str = self.__master_data_management_service.get_season_field_unique_id(season_field_id)

        return self.__analytics_fabric_service.get_metrics(season_field_unique_id, schema_id, start_date, end_date)

    def push_metrics(self, polygon: str,
                     schema_id: str,
                     values: dict):
        """Push metrics in Analytics Fabrics

        Args:
            polygon : A string representing the polygon.
            schema_id : The schema on which to save
            values : Dict representing values to push

        Returns:
            A response object.
        """
        season_field_id: str = self.__master_data_management_service.extract_season_field_id(polygon)

        return self.__analytics_fabric_service.push_metrics(season_field_id, schema_id, values)

    ###########################################
    #           MASTER DATA MANAGEMENT        #
    ###########################################

    def get_available_crops(self):
        """Build the list of available crop codes for the connected user in an enum

        Returns:
            crop_enum: an Enum containing all available crop codes of the connected user
        """
        # get crop code list
        result = self.__master_data_management_service.get_available_crops_code()

        # build an enum with all available crop codes for the connected user
        crop_enum = Enum('CropEnum',
                         {crop['code'] if not crop['code'][0].isdigit() else '_' + crop['code']: crop['code'] for crop
                          in result})

        return crop_enum

    def get_available_permissions(self):
        """Build the list of available permissions codes for the connected user in an enum

        Returns:
            permissions: a string array containing all available permissions of the connected user
        """
        # get crop code list
        result = self.__master_data_management_service.get_permission_codes()

        # build a string array with all available permission codes for the connected user
        permissions = result["permissions"]

        return permissions

    ###########################################
    #           AGRIQUEST                     #
    ###########################################
    def get_agriquest_weather_block_data(self,
                                         start_date: str,
                                         end_date: str,
                                         block_code: AgriquestBlocks,
                                         weather_type: AgriquestWeatherType
                                         ):
        """Retrieve data on all AMU of an AgriquestBlock for the specified weather indicator.

               Args:
                   start_date (str): The start date to retrieve data (format: 'YYYY-MM-dd')
                   end_date (str): The end date to retrieve data (format: 'YYYY-MM-dd')
                   block_code (AgriquestBlocks): The AgriquestBlock name (Enum)
                   weather_type (AgriquestWeatherType) : The Agriquest weather indicator to retrieve (Enum)

               Returns:
                   result ('dataframe'):  pandas dataframe
               """
        # date convert
        start_datetime = datetime.strptime(start_date, "%Y-%m-%d").date()
        end_datetime = datetime.strptime(end_date, "%Y-%m-%d").date()

        # check if the block is dedicated to France
        isFrance = self.__agriquest_service.is_block_for_france(block_code)

        # build the weather indicator list
        weather_indicators = self.__agriquest_service.weather_indicators_builder(start_datetime, end_datetime, isFrance)

        # call the weather endpoint to retrieve data
        result = self.__agriquest_service.get_agriquest_block_weather_data(start_date=start_date, end_date=end_date,
                                                                           block_code=block_code,
                                                                           indicator_list=weather_indicators,
                                                                           weather_type=weather_type)

        return result

    def get_agriquest_ndvi_block_data(self,
                                      day_of_measure: str,
                                      block_code: AgriquestBlocks,
                                      commodity_code: AgriquestCommodityCode
                                      ):
        """Retrieve data on all AMU of an AgriquestBlock for NDVI index

               Args:
                   day_of_measure (str) : The date of measure (format: 'YYYY-MM-dd')
                   block_code (AgriquestBlocks) : The AgriquestBlock name (Enum)
                   commodity_code (AgriquestCommodityCode) : The commodity code (Enum)
               Returns:
                   result ('dataframe'):  pandas dataframe result
               """

        # call the weather endpoint to retrieve data, indicator of NDVI = 1
        result = self.__agriquest_service.get_agriquest_block_ndvi_data(date=day_of_measure, block_code=block_code,
                                                                        commodity=commodity_code, indicator_list=[1])

        return result

    ###########################################
    #           ANALYTICS PROCESSOR           #
    ###########################################

    def get_mr_time_series(self,
                           polygon,
                           start_date: str = "2010-01-01",
                           end_date=None,
                           list_sensors=["micasense", "sequoia", "m4c", "sentinel_2",
                                         "landsat_8", "landsat_9", "cbers4", "kazstsat",
                                         "alsat_1b", "huanjing_2", "deimos", "gaofen_1", "gaofen_6",
                                         "resourcesat2", "dmc_2", "landsat_5", "landsat_7",
                                         "spot", "rapideye_3a", "rapideye_1b"],
                           denoiser: bool = True,
                           smoother: str = "ww",
                           eoc: bool = True,
                           aggregation: str = "mean",
                           index: str = "ndvi",
                           raw_data: bool = False
                           ):

        """Retrieve mr time series on the collection targeted.

        Args:
            start_date : The start date of the time series
            end_date : The end date of the time series
            list_sensors : The Satellite Imagery Collection targeted
            denoiser : A boolean value indicating whether a denoising operation should be applied or not.
            smoother : The type or name of the smoothing technique or algorithm to be used.
            eoc : A boolean value indicating whether the "end of curve" detection should be performed.
            func : The type or name of the function to be applied to the data.
            index : The type or name of the index used for data manipulation or referencing
            raw_data : A boolean value indicating whether the data is in its raw/unprocessed form.
            polygon : A string representing a polygon.

        Returns:
            string : s3 bucket path
        """
        task_id = self.__analytics_processor_service.launch_mr_time_series_processor(
            start_date=start_date,
            end_date=end_date,
            polygon=polygon,
            raw_data=raw_data,
            denoiser=denoiser,
            smoother=smoother,
            aggregation=aggregation,
            list_sensors=list_sensors,
            index=index,
            eoc=eoc,


        )

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        return self.__analytics_processor_service.get_s3_path_from_task_and_processor(task_id, processor_name="mrts")

    def get_harvest_analytics(self,
                              season_duration: int,
                              season_start_day: int,
                              season_start_month: int,
                              crop: Enum,
                              year: int,
                              geometry: str,
                              harvest_type: Harvest):
        """launch a harvest analytics processor and get the metrics in a panda dataframe object

            Args:
                season_duration (int): the duration of the season in days,
                season_start_day (int): the start day value (1 - 31),
                season_start_month (int): the start month value (1 - 12),
                crop (Enum): the geosys crop code,
                year (int): the year value,
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),
                harvest_type (Harvest): the type of Harvest analytics (INSEASON/HISTORICAL)

            Returns:
                A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_harvest_processor(
            season_duration=season_duration,
            season_start_day=season_start_day,
            season_start_month=season_start_month,
            seasonfield_id=sf_unique_id,
            geometry=geometry,
            crop=crop.value,
            year=year,
            harvest_type=harvest_type
        )

        self.logger.info(f"Task Id: {task_id}")

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        # Analytics Schema
        if harvest_type == Harvest.HARVEST_IN_SEASON:
            schema = "INSEASON_HARVEST"
        else:
            schema = "HISTORICAL_HARVEST"

        # if task successfully completed, get metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)

    def get_emergence_analytics(self,
                                season_duration: int,
                                season_start_day: int,
                                season_start_month: int,
                                crop: Enum,
                                year: int,
                                geometry: str,
                                emergence_type: Emergence):
        """launch an emergence analytics processor and get the metrics in a panda dataframe object

            Args:
                season_duration (int): the duration of the season in days,
                season_start_day (int): the start day value (1 - 31),
                season_start_month (int): the start month value (1 - 12),
                crop (Enum): the crop code,
                year (int): the year value,
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),
                emergence_type (Emergence): the type of Emergence analytics (INSEASON/HISTORICAL/DELAY)

            Returns:
                A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_emergence_processor(
            season_duration=season_duration,
            season_start_day=season_start_day,
            season_start_month=season_start_month,
            seasonfield_id=sf_unique_id,
            geometry=geometry,
            crop=crop.value,
            year=year,
            emergence_type=emergence_type
        )

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        # Analytics Schema
        if emergence_type == Emergence.EMERGENCE_IN_SEASON:
            schema = "INSEASON_EMERGENCE"
        elif emergence_type == Emergence.EMERGENCE_HISTORICAL:
            schema = "HISTORICAL_EMERGENCE"
        else:
            schema = "EMERGENCE_DELAY"

        # if task successfully completed, get metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)

    def get_brazil_crop_id_analytics(self,
                                     start_date: str,
                                     end_date: str,
                                     season: CropIdSeason,
                                     geometry: str):
        """launch a brazil-in-season-crop-id analytics processor and get the metrics in a panda dataframe object

            Args:
                start_date (str) : the start date used for the request (format YYYY-MM-dd)
                end_date (str) : the end date used for the request (format YYYY-MM-dd)
                season (CropIdSeason): the season name,
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON)

            Returns:
                A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_brazil_in_season_crop_id_processor(
            start_date=start_date,
            end_date=end_date,
            seasonfield_id=sf_unique_id,
            geometry=geometry,
            season=season.value
        )

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        # Analytics Schema
        schema = "CROP_IDENTIFICATION"

        # if task successfully completed, get metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)

    def get_potential_score_analytics(self,
                                      end_date: str,
                                      nb_historical_years: int,
                                      season_duration: int,
                                      season_start_day: int,
                                      season_start_month: int,
                                      sowing_date: str,
                                      crop: Enum,
                                      geometry: str):
        """launch a potential score analytics processor and get the metrics in a panda dataframe object

            Args:
                season_duration (int): the duration of the season in days,
                season_start_day (int): the start day value (1 - 31),
                season_start_month (int): the start month value (1 - 12),
                crop (Enum): the crop code,
                end_date (str): end date used to calculate potential score
                sowing_date (str): sowing date of the filed used to calculate potential score
                nb_historical_years (int): number of historical years data to calculate potential score
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON)

            Returns:
                A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_potential_score_processor(
            end_date=end_date,
            nb_historical_years=nb_historical_years,
            sowing_date=sowing_date,
            season_duration=season_duration,
            season_start_day=season_start_day,
            season_start_month=season_start_month,
            seasonfield_id=sf_unique_id,
            geometry=geometry,
            crop=crop.value
        )

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        # Analytics Schema
        schema = "POTENTIAL_SCORE"

        # if task successfully completed, get metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)

    def get_greenness_analytics(self,
                                start_date: str,
                                end_date: str,
                                sowing_date: str,
                                crop: Enum,
                                geometry: str):
        """launch a greenness analytics processor and get the metrics in a panda dataframe object

                    Args:
                        start_date (str) : the start date used for the request (format YYYY-MM-dd)
                        end_date (str) : the end date used for the request (format YYYY-MM-dd)
                        sowing_date(str): sowing date of the field used to calculate potential score
                        crop (Enum): the crop code,
                        geometry (str): the geometry to calculate the analytic (WKT or GeoJSON)

                    Returns:
                        A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_greenness_processor(
            start_date=start_date,
            end_date=end_date,
            sowing_date=sowing_date,
            seasonfield_id=sf_unique_id,
            geometry=geometry,
            crop=crop.value
        )

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        # Analytics Schema
        schema = "GREENNESS"

        # if task successfully completed, get metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)

    def get_harvest_readiness_analytics(self,
                                        start_date: str,
                                        end_date: str,
                                        sowing_date: str,
                                        crop: Enum,
                                        geometry: str):
        """launch a harvest readiness analytics processor and get the metrics in a panda dataframe object

            Args:
                start_date (str) : the start date used for the request (format YYYY-MM-dd)
                end_date (str) : the end date used for the request (format YYYY-MM-dd)
                sowing_date(str): sowing date of the field used to calculate potential score
                crop (Enum): the crop code,
                geometry (str): the geometry to calculate the analytic (WKT or GeoJSON)

            Returns:
                A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_harvest_readiness_processor(
            start_date=start_date,
            end_date=end_date,
            sowing_date=sowing_date,
            seasonfield_id=sf_unique_id,
            geometry=geometry,
            crop=crop.value
        )

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        # Analytics Schema
        schema = "HARVEST_READINESS"

        # if task successfully completed, get metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)

    def get_planted_area_analytics(self,
                                   start_date: str,
                                   end_date: str,
                                   geometry: str):
        """launch a planted area analytics processor and get the metrics in a panda dataframe object

                    Args:
                        start_date (str) : the start date used for the request (format YYYY-MM-dd)
                        end_date (str) : the end date used for the request (format YYYY-MM-dd)
                        geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),
                    Returns:
                        A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_planted_area_processor(start_date, end_date, sf_unique_id)
        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        schema = "PLANTED_AREA"

        # if task successfully completed, get latests metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)


    def get_zarc_analytics(self,
                           start_date_emergence: str,
                           end_date_emergence: str,
                           nb_days_sowing_emergence: int,
                           crop: Enum,
                           soil_type: ZarcSoilType,
                           cycle: ZarcCycleType,
                           geometry: str):
        """launch a zarc analytics processor and get the metrics in a panda dataframe object

                        Args:
                            start_date_emergence (str) : the emergence start date used for the request (format YYYY-MM-dd)
                            end_date_emergence (str) : the emergence end date used for the request (format YYYY-MM-dd)
                            nb_days_sowing_emergence (int): the number of days for sowing emergence
                            crop (Enum): the zarc crop code,
                            soil_type (ZarcSoilType): the zarc soil type (1/2/3),
                            cycle (ZarcCycleType): the zarc cycle type (1/2/3),
                            geometry (str): the geometry to calculate the analytic (WKT or GeoJSON),
                        Returns:
                            A Pandas DataFrame containing several columns with metrics
        """
        # validate and convert the geometry to WKT
        geometry = Helper.convert_to_wkt(geometry)

        if geometry is None:
            raise ValueError("The geometry is not a valid WKT of GeoJson")

        # get municipio id from geometry
        municipio_id = self.__gis_service.get_municipio_id_from_geometry(geometry)

        if municipio_id == 0:
            raise ValueError(f"No municipio id found for this geometry")

        # Create seasonfield from geometry and extract uniqueId
        sfd_public_id = self.__master_data_management_service.extract_season_field_id(geometry)
        sf_unique_id = self.__master_data_management_service.get_season_field_unique_id(sfd_public_id)

        task_id = self.__analytics_processor_service.launch_zarc_processor(
            start_date_emergence=start_date_emergence,
            end_date_emergence=end_date_emergence,
            crop=crop.value,
            cycle=cycle.value,
            soil_type=soil_type.value,
            municipio=municipio_id,
            nb_days_sowing_emergence=nb_days_sowing_emergence,
            seasonfield_id=sf_unique_id
        )

        # check the task status to continue or not the process
        self.__analytics_processor_service.wait_and_check_task_status(task_id)

        # Analytics Schema
        schema = "ZARC"

        # if task successfully completed, get metrics from analytics fabric
        return self.__analytics_fabric_service.get_lastest_metrics(sf_unique_id, schema)



__all__ = ["agriquest_service", "analytics_processor_service", "analytics_fabric_service",
           "master_data_management_service", "weather_service", "vegetation_time_series_service",
           "map_product_service"]



import re
import json
from shapely import wkt
from shapely.geometry import shape


class Helper:

    @staticmethod
    def get_matched_str_from_pattern(pattern: str,
                                     text: str) -> str:
        """Returns the first occurence of the matched pattern in text.

        Args:
            pattern : A string representing the regex pattern to look for.
            text : The text to look into.

        Returns:
            A string representing the first occurence in text of the pattern.

        """
        p = re.compile(pattern)
        return p.findall(text)[0]

    @staticmethod
    def convert_to_wkt(geometry):
        """ convert a geometry (WKT or geoJson) to WKT
        Args:
            geometry : A string representing the geometry (WKT or geoJson)

        Returns:
            a valid WKT

        """

        try:
            # check if the geometry is a valid WKT
            if Helper.is_valid_wkt(geometry):
                # return the wkt
                return geometry
        except:
            try:
                # check if the geometry is a valid geoJson
                geojson_data = json.loads(geometry)
                geom = shape(geojson_data)
                geometry = geom.wkt

                return geometry

            except ValueError:
                # geometry is not a valid geoJson
                return None

    @staticmethod
    def is_valid_wkt(geometry):
        """ check if the geometry is a valid WKT
        Args:
            geometry : A string representing the geometry

        Returns:
            boolean (True/False)

        """
        try:
            wkt.loads(geometry)
            return True
        except ValueError:
            return False